<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;

class MakeService extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'make:service {servicename}  {--m|model?=}';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Create a service class for any given model';

    /**
     * Execute the console command.
     *
     * @return int
     */


     const STUB_TEMPLATE = 'app/Services/ServiceStub.php';


     public function handle()
    {
        $eol = PHP_EOL;

        $className = $this->argument('servicename');

        $tempFilename = 'app/Services/' . $className;
        
        $filename = $tempFilename . '.php';


        if (!touch($filename))  $this->error($eol . 'Error Occurred' . $eol);
        
        $generatedStub = $this->generateServiceStub($className);

        $newServiceCreated = file_put_contents($filename, $generatedStub);


        if (!$newServiceCreated) $this->error($eol . 'Error Occurred' . $eol);

        $this->info($eol . "Service [ $filename ] was created successfully!" . $eol);

        return Command::SUCCESS;
    }



    /**
     * generates a stub for a new service.php file
     *
     * @param [type] $newfile
     * @return string
     */
    private function generateServiceStub($newClassName): string
    {
        // $service_stub = 'app/Services/' . 'ServiceStub.php';
        
        $stub_contents = file_get_contents(self::STUB_TEMPLATE);

        $freshServiceContents = str_replace('ServiceStub', $newClassName, $stub_contents);
        
        return $freshServiceContents;
    }
}
