<?php

use App\Models\Book;
use App\Models\Order;
use App\Models\Author;
use App\Models\Developer;
use App\Models\Organisation;
use Illuminate\Support\Facades\Route;
use Illuminate\Database\Eloquent\Model;
use Gloudemans\Shoppingcart\Facades\Cart;
use App\Services\SchoolIncomeService as SIS;
use Illuminate\Database\Eloquent\Relations\Relation;

// global variables
// $session_checks =

if (!function_exists('authy')) {
    /**
     * Get the authenticated user
     *
     * @return Illuminate\Contracts\Auth\Authenticatable|null
     */
    function authy()
    {
        return auth()->user();
    }
}



if (!function_exists('author')) {
    /**
     * Get the authenticated author
     *
     * @return Relation|Model
     */
    function author($queryBuilder = false): Relation|Author
    {
        return $queryBuilder ? auth()->user()->author() : auth()->user()->author;
    }
}


if (!function_exists('reqy')) {
    /**
     * Get the request user
     *
     * @return Illuminate\Contracts\Auth\Authenticatable|null
     */
    function reqy()
    {
        return request()->user();
    }
}


if (!function_exists('getBookNameFromSlug')) {
    /**
     * Get the book name from any given slug
     *
     * @return Illuminate\Contracts\Auth\Authenticatable|null
     */
    function getBookNameFromSlug($slug)
    {
        return $slug ? Book::whereSlug($slug)->first()->name : '';
    }
}



if (!function_exists('generateTxRef')) {

    /**
     * returns a 16 digit tx_ref for any new order (for use in any payment gateway service)
     */
    function generateTxRef()
    {
        $variableRand = rand(0, time());
        $randomFirst9Digits = substr(uniqid(), 4);
        $randomlast7Digits = substr($variableRand, 0, 7);

        return $uniqueRandomIdentifier = 'TRX' . substr(($randomFirst9Digits . $randomlast7Digits), 0, 16);
    }
}


if (!function_exists('endOfUrl')) {
    /**
     * returns the end of a url
     * @return string
     */

    function endOfUrl($url)
    {
        // $resBody['paymentUrl'] = "https://checkout-dev.globalaccelerex.com/pay/17079048CrEub5AzRQ";
        $urlArray = parse_url($url, PHP_URL_PATH);
        $segments = explode('/', $urlArray);
        return $endOfPaymentUrl = end($segments);
    }
}


if (!function_exists('roundUpToNearestTens')) {
    /**
     * Round up a number to the nearest future tens. 
     * 
     * This will always round UP a number to the nearest tens
     * eg. 11 is rounded to 20.
     *
     * @return int
     */
    function roundUpToNearestTens($newPrice): int
    {
        $roundValue = round($newPrice, -1);

        if ($roundValue < $newPrice)
            $newPrice += 5;

        return $newPrice = $roundValue = round($newPrice, -1);
    }
}





if (!function_exists('dev')) {
    /**
     * Get Developer instance for the authenticated user
     *
     * @return Developer
     */
    function dev()
    {
        return auth()->user()->dev();
    }
}

if (!function_exists('sessionHasFlashData')) {
    /**
     * Checks if data was flashed to the session
     */
    function sessionHasFlashData(): bool
    {
        // if(  session()->has('warning') || session()->has('danger') ||  session()->has('error') ||
        // session()->has('success') || session()->has('status'))

        [
            'sess_danger' => $sess_danger,
            'sess_error' => $sess_error,
            'sess_warning' => $sess_warning,
            'sess_success' => $sess_success,
            'sess_status' => $sess_status
        ] = defineSessionVars();

        if ($sess_warning || $sess_danger || $sess_error || $sess_success || $sess_status) {
            return true;
        }

        return false;
    }
}

if (!function_exists('defineSessionVars')) {
    function defineSessionVars()
    {
        $sess_warning = session()->has('warning');
        $sess_danger = session()->has('danger');
        $sess_error = session()->has('error');
        $sess_success = session()->has('success');
        $sess_status = session()->has('status');

        return compact('sess_warning', 'sess_danger', 'sess_error', 'sess_success', 'sess_status');
    }
}

if (!function_exists('getFlashedDataForSwal')) {

    function getFlashedDataForSwal()
    {
        if (sessionHasFlashData()) {
            return obtainFlashData();
        }
    }
}

if (!function_exists('getViteFiles')) {

    /**
     * getViteFiles
     * Returns the resources used by vite 
     * @return array
     */
    function getViteFiles(): array
    {
        return ['resources/css/app.css', 'resources/scss/app.scss', 'resources/js/app.js'];
    }
}


if (!function_exists('internetConnection')) {
    /**
     * Ascertains if the client pc is connected to the internet
     * This function returns 1 if internet connection exists 
     *  or 0 if no internet connection
     * @return int  true if internet connection exists 
     */
    function internetConnection(): int
    {
        $connected = @fsockopen("www.example.com", 80);
        if ($connected) {
            fclose($connected);
            return 1;
        } else
            return 0;
    }

}

if (!function_exists('getSchoolVAT')) {

    /**
     * gets the default school vat = 0 in test mode or 100 in live mode
     */
    function getSchoolVAT(): int
    {
        return env('APP_PAYMENT_MODE') == 'TEST' ? 0 : 100;
    }
}




if (!function_exists('durationRepo')) {

    function durationRepo(): array
    {
        return ['today', 'this_week', 'this_month', 'this_year', 'yesterday', 'last_week', 'last_month', 'last_year'];
    }
}


if (!function_exists('obtainFlashData')) {

    function obtainFlashData()
    {
        [
            'sess_danger' => $sess_danger,
            'sess_error' => $sess_error,
            'sess_warning' => $sess_warning,
            'sess_success' => $sess_success,
            'sess_status' => $sess_status,
        ] = defineSessionVars();

        $icon = $title = $generateMessage = '';

        if ($sess_warning) {
            $icon = 'warning';
            $title = 'Oops!';
            // dd($icon, $title, session('warning'), is_array(session('warning')));
            if (is_array(session('warning'))) {
                $warningArray = session('warning');
                $icon = 'warning';
                $title = 'Welcome ' . $warningArray['salutation']; //Welcome ma
                // session('warning',  $warningArray['message']);
                $generateMessage = $warningArray['message'];
            } else {
                $generateMessage = session('warning');
            }
            $warningMessage = $generateMessage;
        } elseif ($sess_success) {
            $generateMessage = session('success');
            if (is_array(session('success'))) {
                $successArray = session('success');
                $icon = 'success';
                $title = 'Welcome ' . $successArray['salutation']; //Welcome ma
                // session('success',  $successArray['message']);
                $generateMessage = $successArray['message'];
            } else {
                $icon = 'success';
                $title = 'Good job!';
                $generateMessage = session('success');
            }
            $successMessage = $generateMessage;
            //   dd($successArray, $successArray['message'], 'the session variable', session('success'), $successMessage);
        } elseif (session()->has('info') or $sess_status) {
            $icon = $title = 'info';
            $generateMessage = $infoMessage = session('info') ?? session('status');
        } elseif ($sess_error or $sess_danger) {
            $icon = 'error';
            $title = 'Oops!';
            $generateMessage = $errorMessage = session('error') ?? session('danger');
        }
        //  $message = session('warning') ?? (session('success') ?? (session('error') ?? (session('danger') ?? session('status'))));
        $message = $generateMessage;

        return compact('icon', 'message', 'title');
    }
}


if (!function_exists('dbLike')) {
    /**
     * dbLike represents the case-insensitive 'like' operator for sql query (mariadb/postgres)
     *
     * @return string
     */
    function dbLike(): string
    {
        /**
         * @var string $like represents the case-insensitive 'like' operator for sql query
         */
        return $like = (env('DB_CONNECTION') == 'mysql') ? 'like' : 'ilike'; //mariadb vs pgsql
    }
}

if (!function_exists('formatPath')) {

    /**
     * formats the file path of file, for easy storage and retrieval from the db
     * Thsi function silently removes the '/storage/' from all file paths, so that they can be referenced via their respective disks
     */
    function formatPath(string|null $path)
    {
        return $path ? str_replace('/storage/', '', $path) : '';
    }
}

if (!function_exists('getCartItemByID')) {

    function getCartItemByID(int $id): object|null
    {
        return Cart::search(function ($cartItem, $rowId) use ($id) {
            return (int) $cartItem->id === (int) $id;
        })->first();
    }
}

if (!function_exists('slugFrom')) {

    /**
     * Create a slug from a name by replacing spaces ' ' with underscores '_'
     * the resulting string is a lowercase string
     *
     * @return string
     */
    function slugFrom(string $orgName, $separator = '_'): string|bool
    {
        if (isset($separator) and !in_array($separator, ['_', '-']))
            return false;

        return !$separator
            ? strtolower(str_replace(' ', '_', $orgName))
            : strtolower(str_replace(' ', $separator, $orgName));
    }
}


if (!function_exists('orgSlugFrom')) {

    /**
     * Create an organisation slug name from an IMS code
     * 
     *
     * @return string
     */
    function orgSlugFrom(string $imsCode): string
    {
        //for slug: remove the last two or more chars which is = '_$id' eg. '_232'
        return explode('_', $imsCode)[0];
    }
}



if (!function_exists('profitMargin')) {

    /**
     * Returns a the profit margin for the specified purchase type 
     *@param string|int $type  represents the purchase type can only be 'ebook' or 'hardcover'
     *
     *  values for 'ebook' => [1, 'ebook']
     *
     *   values for 'hardcover' => ['hard cover' , 'hardcover' ,0, null]
     * @return float
     */
    function profitMargin(string $type = null): float
    {
        return match ($type) {
            'ebook', '1' => 1, //represents 100%
            'hardcover', 'hard cover' => 0.25, //represents 25%
            default => 0.25 //represents 25% for hardcopy
        };
    }
}

if (!function_exists('createOrgIMSCodeFrom')) {

    /**
     * Create or Generate a unique IMS Code from a new org name by replacing spaces ' ' with underscores '_'
     * the resulting string is a lowercase string
     *
     * @return string
     */
    function createOrgIMSCodeFrom(string $orgName): string
    {
        // return strtolower(str_replace(' ', '_', $orgName));
        $arrayOfStrings = explode(' ', $orgName);
        $firstLetters = '';
        foreach ($arrayOfStrings as $string) {
            if (in_array($string, ['of', 'and', 'in', 'for']))
                continue;
            $firstLetters .= $string[0];
        }
        $slugID = Organisation::all()->max('id') + 1;
        $slug = "{$firstLetters}_{$slugID}";
        return strtoupper($slug);
    }
}


if (!function_exists('generateRandomString')) {

    /**
     * Generate a random string
     * @return bool|string
     */
    function generateRandomString(string $length = null): bool|string
    {
        return substr(fake()->shuffle('abcdefghijklmnopqrstuvwxyz'), 0, $length);
    }
}




if (!function_exists('sis')) {

    /**
     * Get a SchoolIincomeService  instance
     * @param string $slug
     * @return SIS
     */
    function sis(string $slug = null): SIS
    {
        return new SIS($slug);
    }
}



if (!function_exists('generateRandomOrgName')) {

    /**
     * Generate a random organisation name
     * @param string $organisationType can be 'uni', 'poly', 'college'
     * 'uni' represents university
     * 'poly' represents polytechnic
     * 'college' represents college of education
     * 'unis' represents University of $xxx
     * @return bool|string
     */
    function generateRandomOrgName(string $organisationType = 'unis'): bool|string
    {
        $suffix = match ($organisationType) {
            'uni' => ' University',
            'college' => ' College of Education',
            'poly' => ' Polytechnic',
            default => '',
        };

        $prefix = $organisationType == 'unis' ? 'University of ' : '';
        return $prefix . fake()->city() . $suffix;
        // return $orgName = $prefix ? $prefix . fake()->city() : fake()->city() . $suffix;
    }
}

if (!function_exists('money')) {

    /**
     * Get the current Route name 
     * alias for number_format()
     *
     * @return string
     */
    function money(int|float $amount = null): string
    {
        return number_format($amount, 2);
    }
}

if (!function_exists('thisRouteName')) {

    /**
     * Get the current Route name 
     * alias for Route::currentRouteName()
     *
     * @return bool|string
     */
    function thisRouteName(string $routeName = null): bool|string
    {
        return Route::currentRouteName();
    }
}


if (!function_exists('isMyRoute')) {

    /**
     * Checks if the current Route is the same as provided route name 
     *
     * @return bool|string
     */
    function isMyRoute(string $routeName): bool|string
    {
        return $routeName === Route::currentRouteName();
    }
}



if (!function_exists('getAllRouteNames')) {

    /**
     * Get All Route names
     *
     * @return array
     */
    function getAllRouteNames(): array
    {
        $routeNames = [];

        foreach (Route::getRoutes()->getRoutes() as $route)
            $routeNames[] = $route->getName();

        return $routeNames;
    }
}


if (!function_exists('userRole')) {
    /**
     * Get the current Auth::user() Role or compare it against a given role
     *
     * @return bool|string
     */
    function userRole(string $userRole): bool|string
    {
        return $userRole ? $userRole == authy()->role : authy()->role;
    }
}


if (!function_exists('getAllConfigRoutes')) {

    /**
     * Get All Config Route names
     * These config routes refer to the routes used to modify user data in the application.
     * @alias configRoutes
     * @return array
     */
    function getAllConfigRoutes(): array
    {
        $configRoutes = [];

        foreach (getAllRouteNames() as $routeName)
            if (str_contains($routeName, '-dept'))
                $configRoutes[] = $routeName;

        array_push($configRoutes, 'user.config', 'profile.edit'); // add the config and the profile.edit route

        return $configRoutes;
    }
}


if (!function_exists('getAllBookRoutes')) {

    /**
     * Get All Config Route names
     * These config routes refer to the routes used to modify user data in the application.
     * @alias configRoutes
     * @return array
     */
    function getAllBookRoutes(): array
    {
        $bookRoutes = [];

        foreach (getAllRouteNames() as $routeName)
            if (str_contains($routeName, 'books.'))
                $bookRoutes[] = $routeName;

        return $bookRoutes;
    }
}


if (!function_exists('generateMatNumFmtForDSPG')) {
    /**
     * generateMatNumFmtForDSPG
     * this function should be based on the institution
     * every institution should have a custom class built for them
     * that will handle these kinds of operations because the 
     * mat number fomats varies across different schools
     * @param  mixed $matNum represents the sample mat_num
     * @param  mixed $separator represents the separator used in the matnum for this school
     * @return string is the formatted or derived mat num format for any given dept in that school
     */
    function generateMatNumFmtForDSPG($matNum, $separator = '/')
    {
        $matNumAsArray = explode($separator, $matNum);
        // remove the last part
        $studentUniqueMatricId = array_pop($matNumAsArray);
        // re convert to string
        $matNumFormat = implode($separator, $matNumAsArray);

        return $matNumFormat;
    }
}


if (!function_exists('generateMatNumFmtForDSPGA')) {
    /**
     * generateMatNumFmtForDSPG
     * this function should be based on the institution
     * every institution should have a custom class built for them
     * that will handle these kinds of operations because the 
     * mat number fomats varies across different schools
     * @param  mixed $matNum represents the sample mat_num
     * @param  mixed $separator represents the separator used in the matnum for this school
     * @return string is the formatted or derived mat num format for any given dept in that school
     */
    function generateMatNumFmtForDSPGA($matNum, $separator = '/')
    {
        return generateMatNumFmtForDSPG($matNum);
    }
}


if (!function_exists('generateMatNumFmtForDSPGO')) {
    /**
     * generateMatNumFmtForDSPGO
     * this function should be based on the institution
     * every institution should have a custom class built for them
     * that will handle these kinds of operations because the 
     * mat number fomats varies across different schools
     * @param  mixed $matNum represents the sample mat_num
     * @param  mixed $separator represents the separator used in the matnum for this school
     * @return string is the formatted or derived mat num format for any given dept in that school
     */
    function generateMatNumFmtForDSPO($matNum, $separator = '/')
    {
        return generateMatNumFmtForDSPG($matNum);
    }
}



if (!function_exists('generateMatNumFmtForLASUED')) {
    /**
     * generateMatNumFmtForLASUED
     * this function should be based on the institution
     * every institution should have a custom class built for them
     * that will handle these kinds of operations because the 
     * mat number fomats varies across different schools
     * @param  mixed $matNum represents the sample mat_num
     * @param  mixed $separator represents the separator used in the matnum for this school
     * @return string is the formatted or derived mat num format for any given dept in that school
     */
    function generateMatNumFmtForLASUED($matNum, $separator = '/')
    {
        return generateMatNumFmtForDSPG($matNum);
    }
}

if (!function_exists('matchMatNum')) {

    /**
     * matchMatNum
     * Match Mat number to department.
     * this helps to determine if a student/client belongs to a specific department
     * It also helps in generating reports for that department
     * 
     * @param  mixed $client refers to the client being checked
     * @param  mixed $dept refers to the department (haystack) where client is being looked for
     * @return bool if client is found in department
     */
    function matchMatNum($client, $dept): bool
    {
        $allParts = explode('/', $client->client_model->mat_num); // for mat num

        $allSampleParts = explode('/', $dept->sample_matric_num); // for dept sample mat num

        array_pop($allParts); //remove last part of the mat num
        $fixedParts = strtolower(implode('/', $allParts));

        array_pop($allSampleParts); //remove last part of the samplemat num
        $fixedSampleParts = strtolower(implode('/', $allSampleParts));

        return $fixedParts == $fixedSampleParts;
    }
}


if (!function_exists('updateInventory')) {
    /**
     * update inventory count for hard cover purchases
     */
    function updateInventory(Order $order)
    {
        foreach ($order->getBasket() as $orderItem) {
            // search for hardcover items in the cart/basket
            if ($orderItem->options->type !== 'ebook') {
                $relatedModel = Book::find($orderItem->id);
                // we can only purchase one type & qty of any given book per order.
                //  eg. 1 ebook per order
                $relatedModel->inventory -= 1;
                $relatedModel->save();
            }
        }
    }

}
