<?php

namespace App\Http\Controllers\Auth;

use App\Enums\UserRoleEnum;
use Illuminate\View\View;
use App\Models\Organisation;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Http\RedirectResponse;
use Illuminate\Auth\Events\Registered;
use App\Providers\RouteServiceProvider;
use App\Http\Requests\UserSignupRequest;
use Symfony\Component\Console\Input\Input;

class RegisteredUserController extends Controller
{
    /**
     * Display the registration view.
     */
    public function create(): View
    {
        $orgs = Organisation::all();
        return view('snit.auth.register-basic', compact('orgs'));
    }

    /**
     * Handle an incoming registration request.
     *
     * @throws \Illuminate\Validation\ValidationException
     */
    public function store(UserSignupRequest $request): RedirectResponse
    {
        //Prerequisites: post validation after main UserSignupRequest validation
        $this->validate($request, [
            'name' => 'required',
            'email' => 'required',
            'role' => 'required',
            'password' => 'required|confirmed',
        ]);
        // dd('from controller', $request->errors);

        if ($request->errors) {
            return redirect('/register')->with('error', $request->errors[0])->withInput();
        }
        $user = $request->createUser();

        event(new Registered($user));

        Auth::login($user);

        return $request->role == UserRoleEnum::ORG
            ? redirect(RouteServiceProvider::SCH_HOME)
            : redirect(RouteServiceProvider::HOME);
    }
}
