<?php

namespace App\Http\Controllers;

use App\Http\Requests\StoreBookRequest;
use App\Models\Book;
use App\Models\Publisher;
use App\Services\Book\BookService;
use Illuminate\Contracts\View\View;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Storage;

class BookController extends Controller
{

    public function __construct(protected BookService $service)
    {
    }

     /**
     * Display a listing of the resource.
     * Display a listing of all books sold by a particular instructor.
     * Display a listing of the resource.
     */
    public function analysis(): View
    {
        $table_headers = [
            's/n',
            'File',
            'Front <br> cover',
            'inven <br> tory',
            'Title',
            // pricing
            'book price <br> paper | ebook',
            // profit
            'hardcover <br> profit (₦)',
            'Ebook <br> profit (₦)',
            'total <br> profit',

            'Edit <br> Book',
            'Edit <br> File',
            // 'del',
        ];
        // $allBooks = Auth::user()->author->books()->has('doc')->orderBy('id', 'desc')->get();
        $allBooks = Auth::user()->author->books()->orderBy('id', 'desc')->get();
        // dd($allBooks);
        return view('snit.auth.dashboard.books.analysis', compact('allBooks', 'table_headers'));
    }


    /**
     * Display a listing of the resource.
     * Display a listing of all books sold by a particular instructor.
     * Display a listing of the resource.
     */
    public function index(): View
    {
        $table_headers = [
            's/n',
            'File',
            'Front <br> cover',
            'inven <br> tory',
            'Title',
            'ISBN',
            // pricing
            'book price <br> paper | ebook',

            'Edit <br> Book',
            'Edit <br> File',
            // 'del',
        ];
        // $allBooks = Auth::user()->author->books()->has('doc')->orderBy('id', 'desc')->get();
        $allBooks = Auth::user()->author->books()->orderBy('id', 'desc')->get();
        // dd($allBooks);
        return view('snit.auth.dashboard.books.index', compact('allBooks', 'table_headers'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return View
     */
    public function create()
    {
        $this->authorize('create', Book::class);

        $all_publishers = Publisher::all();

        return view('snit.auth.dashboard.books.create', compact('all_publishers'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  StoreBookRequest  $request
     */
    public function store(StoreBookRequest $request): RedirectResponse
    {
        $this->authorize('create', Book::class);
        
        // avoid duplicates
        if ($this->service->bookExists($request)) {
            return back()->with('warning', 'Book Already exists with ISBN: ' . $request['ISBN']);
        }

        if ($bookCreated = $this->service->createBook($request)) {
            $urlOptions = ['uploadfiles' => true, 'book' => $bookCreated];
            return redirect()
                ->route('docs.create', $urlOptions)
                ->with('success', 'Book created successfully');
        }
        return back()->with('warning', 'Error occured');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return Response
     */
    public function show($id)
    {
        dd('show this book page');
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     */
    public function edit(Book $book): View
    {
        $this->authorize('update', $book);

        $all_publishers = Publisher::all();

        return view('snit.auth.dashboard.books.edit', compact('all_publishers', 'book'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  StoreBookRequest  $request
     * @param  int  $id
     */
    public function update(StoreBookRequest $request, Book $book): RedirectResponse
    {
        $this->authorize('update', $book);
        // dd($request->all());

        if (!$book->service->updateBook($request, $book)) {
            return $this->service->redirectWithBookFailedToUpdateMessage();
        }

        if (!$book->doc) {
            return $this->service->redirectToCreateNewDocumentForBook($book);
        }

        if (!$book->doc->front_cover || !Storage::disk('public')->exists($book->doc->front_cover)) {
            return $this->service->redirectToEditDocumentForBook($book);
        }

        return back()->with('success', 'Book Updated successfully');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return RedirectResponse
     */
    public function destroy(Book $book): RedirectResponse | null
    {
        $this->authorize('delete', $book);

        return ($book->service->deleteBook($book));
    }

    // helpers

    public function designated_delete(Book $book)
    {
        return $this->destroy($book);
    }
}

// author->books()->whereName('Tatum Dodson')->get()->count() > 1
