<?php

namespace App\Http\Controllers;

use App\Models\PG;
use App\Models\Client;
use App\Models\Country;
use App\PaymentGatewayServices\TestRexpay;
use Illuminate\View\View;
use App\Models\Organisation;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\PaymentGatewayServices\Rexpay;
use Gloudemans\Shoppingcart\Facades\Cart;
use App\Http\Requests\GuestCheckoutRequest;
use App\PaymentGatewayServices\Flutterwave;
use App\Http\Requests\GuestCheckoutRequestBasic;

class CheckoutController extends Controller
{
    public function __construct()
    {
    }


    /**
     * Handle the incoming request.
     */
    public function index(Request $request): View
    {
        $nations = Country::orderBy('name')->get();
        $orgs = Organisation::orderBy('name')->get();

        return view('snit.guest.checkout.index', compact('nations', 'orgs'));
    }

    public function chooseClient()
    {
        return view('snit.guest.checkout.new_or_old_client_selection_page');
    }

    public function prepareForPayment(GuestCheckoutRequest $request)
    {
        // prevalidation : no payment without mat_num
        $client = null;

        // $request->validate(['mat_num' => 'required|email|exists:clients']);
        $clientExists = Client::query()->where('mat_num', $request->mat_num)?->first();
        
        $client = ($clientExists) ?? Client::create($request->validated());

        $cartTotal = (float) str_replace(',', '', Cart::total()); //save cartTotal before destroying cart

        $identifier = $this->generateUniqueRandomIdentifier();

        $validParams4PaymentInit = ($cartTotal && $identifier && $client);


        /**
         * @var array $params represents the payment parameters for all payment gateways
         */
        $params = $validParams4PaymentInit ? compact('cartTotal', 'identifier', 'client') : null;

        // redirect to a choose payment gateway modal
        if ($params) {
            session(['paymentParams' => $params, 'client' => $client]);
            return redirect()->route('payment-gateway.index'); //page to select payment gateway
        }
        return back()->with('error', 'Oops! Try Again, Could not validate data to initialize payment');
    }


    public function store(Request $request)
    {
        $params = session('paymentParams');
        $foundPGS = PG::whereName($request->payment_gateway)->first();
        $params['pgs_id'] = $foundPGS->id;
        $params['pgs_name'] = $foundPGS->name;
        session()->put('paymentParams', $params); //update session variable

        if ($request->payment_gateway == 'flutterwave') {
            $pendingFlwTrx = $this->paywithFlw();

            return ($pendingFlwTrx['paymentLink'])
                ? redirect($pendingFlwTrx['paymentLink'])
                : back()->with('error', 'Oops! Try Again, Something went wrong');

            // return back()->with('error', "Oops! Flutterwave not supported");
        }

        if ($request->payment_gateway == 'rexpay') {

            $pendingRexPayTrx = $this->paywithRexpay();

            return ($pendingRexPayTrx['paymentLink'])
                ? redirect($pendingRexPayTrx['paymentLink'])
                : back()->with('error', 'Oops! Try Again, Could not create payment');
        }
        return back()->with('error', "Hey! pls don't try this again");
    }

    /**
     * this returns an array consisting of flw payment link and the 'pending' Order model
     */
    public function payWithFlw(): array
    {
        $params = session('paymentParams');

        $flwObj = new Flutterwave;

        $reqBody = $flwObj->buildTrxData();

        $newPendingOrder = $this->createNewPendingOrderForClient($reqBody);

        $response = $flwObj->createPayment($reqBody);

        $paymentLink = $flwObj->prePaymentValidations($response);

        return compact('paymentLink', 'newPendingOrder');
    }


    /**
     * returns an array containing the payment link and the 'pending' Order model
     */
    public function payWithRexpay()
    {
        $params = session('paymentParams');

        // $rexPayObj = (env('APP_PAYMENT_MODE') == "LIVE") ? new Rexpay : new TestRexpay;
        $rexPayObj = new Rexpay;

        $reqBody = $rexPayObj->buildTrxData(); //3. let's build flw data

        $newPendingOrder = $this->createNewPendingOrderForClient($reqBody);

        $response = $rexPayObj->createPayment($reqBody);

        $paymentLink = $rexPayObj->prePaymentValidations($response);

        return compact('paymentLink', 'newPendingOrder');
    }

    public function createNewPendingOrderForClient(array $reqBody)
    {
        $params = session('paymentParams'); //important

        $cartItemsInJson = json_encode(Cart::content());

        // handle trx Reference key in reqBody based on different PGS
        $trxRefKey = $this->getTrxRef4rmPGSReqBody();

        return $newPendingOrder = $params['client']->orders()->updateOrCreate(
            [
                'identifier' => $params['identifier'],
            ],
            [
                'status' => 'pending',

                'tx_ref' => $reqBody[$trxRefKey], // 'tx_ref' for flw, 'reference' for rexpay

                'pgs_tx_id' => 'await_pgs_tx_id_' . uniqid(),
                'content' => $cartItemsInJson, //  check $this->storeCartInDB() for clarity
                'total_amount' => $params['cartTotal'],
                'pgs_id' => $params['pgs_id'],

                //app needs to compute the following
                'authors_remit' => 0.00,
                'pgs_remit' => 0.00,
                'dev_remit' => 0.00,
            ]
        );
    }


    public function getTrxRef4rmPGSReqBody(): string
    {
        $params = session('paymentParams'); //important

        return $ref = match ($params['pgs_name']) {
            'flutterwave' => 'tx_ref',
            'rexpay' => 'reference'
        };
    }


    /**
     * returns a 10 digit unique identifier for the cart
     */
    public function generateUniqueRandomIdentifier()
    {
        $trxRef = str_replace('TRX', '', generateTxRef());

        return substr($trxRef, 0, 10);
    }

}
