<?php

namespace App\Http\Controllers;

use App\Models\Book;
use App\Models\Dept;
use Illuminate\Http\Request;
use PhpParser\Node\Stmt\ElseIf_;
use Illuminate\Support\Collection;
use Illuminate\Contracts\View\View;
use Illuminate\Http\RedirectResponse;

class DeptController extends Controller
{
    /**
     * Display a listing of the resource.
     * 
     * 0 references but is used by Route::resource(); !important
     *
     * @return \Illuminate\Http\Response|View
     */
    public function index(): view
    {
        $allDepartments = reqy()->depts;
        $table_headers = [
            'name',
            'program_type',
            'dept_level',
            'sample_matric_num',
            'institution',
            'edit',
            'delete',
            // 'generate <br> sales <br> report',
        ];

        return view('snit.auth.dashboard.depts.index', compact('allDepartments', 'table_headers'), [
            'dept_levels' => $this->dept_levels,
        ]);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response|View
     */
    public function create(): View
    {
        return view('snit.auth.dashboard.depts.create', [
            'dept_levels' => $this->dept_levels,
            'myBooks' => author()->books,
        ]);


    }

    /**
     * Store a newly created resource in storage.
     *
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request): RedirectResponse
    {
        // dd((collect($request->attached_books))->first());
        $request->validate([
            'attached_books' => 'required',
            'name' => 'required',
            'dept_level' => 'required',
            'sample_matric_num' => 'required',
        ]);

        //important b4 create or it won't work
        $request['org_id'] = author()->org_id;
        // attach author id to the slug so it's unique
        $request['slug'] = slugFrom($request['name']) . "__" . auth()->id();


        $newDept = authy()->depts()->create($request->except('_token'));

        // post creation : very important!
        $this->attachRequestedBooksToDept($request, $newDept);

        $successMessage = "{$newDept->name} Department Registered Successfuly";

        return $newDept
            ? back()->with('success', $successMessage)
            : back()->with('failed', 'Oops! Error Occurred... Please Retry');
    }

    /**
     * Display the specified resource.
     *
     * @return \Illuminate\Http\Response|RedirectResponse
     */
    public function show(Request $request, Dept $dept)
    {

    }

    /**
     * Show the form for editing the specified resource.
     *
     * @return \Illuminate\Http\Response|View
     */
    public function edit(Dept $dept): View
    {
        return view('snit.auth.dashboard.depts.edit', [
            'dept_levels' => $this->dept_levels,
            'dept' => $dept,
            'myBooks' => author()->books,
        ]);

    }

    /**
     * Update the specified resource in storage.
     *
     * @return \Illuminate\Http\Response|RedirectResponse
     */
    public function update(Request $request, Dept $dept): RedirectResponse
    {
        $request->validate([
            'attached_books' => 'nullable',
        ]);

        if (isset($request->attached_books) and count($request->attached_books) > 0)
            $this->attachRequestedBooksToDept($request, $dept, 1);

        $updatedDept = $dept->update($request->except('_token', '_method'));

        return ($updatedDept)
            ? back()->with('success', "{$dept->name} Department Updated Successfuly")
            : back()->with('failed', 'Oops! Error Occurred... Please Retry');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @return \Illuminate\Http\Response|RedirectResponse
     */
    public function destroy(Dept $dept)
    {
        return ($deptDeleted = $dept->delete())
            ? back()->with('success', 'Department Deleted Successfully')
            : back()->with('failed', 'Oops! Error Occurred');
    }

    /**
     * attachRequestedBooksToDept
     *
     * @param  mixed $request
     * @param  mixed $dept
     * @param  mixed $updateOperation resets (deletes all) the previously linked attachments in pivot table for update requests
     * @return void
     */
    private function attachRequestedBooksToDept(Request $request, Dept $dept, bool $updateOperation = false)
    {
        if (boolval($updateOperation) == true)
            $dept->books()->detach(); // clear all book attachments

        $singleAttachedBook = count($request->attached_books) == 1;
        $multipleAttachedBooks = count($request->attached_books) > 1;
        
        if ($singleAttachedBook) {

            $singleBookID = (collect($request->attached_books))->first();
            !empty($singleBookID) ? $this->attachOneBookToDept($singleBookID, $dept) : null;

        } elseif ($multipleAttachedBooks and $multipleBooks = $request->attached_books) {
            // attach multiple books to this dept
            foreach ($multipleBooks as $singleBookID) {
                $singleBookID = Book::find($singleBookID) ? $singleBookID : null;
                $this->attachOneBookToDept($singleBookID, $dept);
                !empty($singleBookID) ? $this->attachOneBookToDept($singleBookID, $dept) : null;
            }
        }

    }


    /**
     * @param int $bookId represents the book Id to be attached to the department
     * @param Dept $dept represents the department you wish to attach the book Id to
     * 
     */
    public function attachOneBookToDept(int $bookId, Dept $dept)
    {
        $bookId = Book::find($bookId) ? $bookId : null;
        // attach one book to this dept
        $dept->books()->attach($bookId);
    }



    /**
     * The page that shows a list of all the clients related to a specific dept
     * this way we can see all those who have paid or bought a book in a given dept
     */
    public function clients_show(Request $request, $deptSlug)
    {
        $table_headers = $this->getTableHeaders();
        $hideFromPrint = $this->getHeadersToHideFromPrint();

        $requestedDept = $request['requestedDept'] = Dept::whereSlug($deptSlug)->first();

        /**
         * $allRelatedClients in this case refers to all departmental clients
         */
        $allRelatedClients = $allClientsInDept = $requestedDept?->getClients();
        $booksSoldInDept = $requestedDept?->getBooksSoldInDept();

        // filter results (ebook or hardcopy) //compulsory filter
        if (isset($request->purchaseType))
            $allClientsInDept = $allClientsInDept->where('itemPurchased.options.type', $request->purchaseType);

        // bookSlug is used to determine if a certain book was requested from the same route via added params
        // to get information about a specific book, we  attach bookSlugs as additional params to same route
        if ($request->bookSlug) {
            // getbook name
            $requestedBookName = author()->books()
                ->where('slug', $request->bookSlug)?->first()?->name;

            // filter results based on bookname
            $allRelatedClients = $allClientsInDept->where('itemPurchased.name', $requestedBookName);
        }

        return view(
            'snit.auth.dashboard.clients.clients_by_dept.show_all',
            compact('allRelatedClients', 'table_headers', 'requestedDept', 'booksSoldInDept', 'hideFromPrint')
        );
    }


    /**
     * Not in use anymore
     * The page that shows a list of all the clients related to a specific book that ordered
     * hardcover books only
     * this way we can see all those who have paid or bought a specific book
     */
    public function clients_hardcover_show(Request $request, $deptSlug)
    {
        $table_headers = $this->getTableHeaders();
        $hideFromPrint = $this->getHeadersToHideFromPrint();

        // NB:$request['requestedDept'] is used in $this->getClientsBasedOnOrderType($request)
        $requestedDept = $request['requestedDept'] = Dept::whereSlug($deptSlug)->first();

        $allDepartmentalClients = $requestedDept->getClients();

        $allRelatedClients = $allDepartmentalClients->where('itemPurchased.options.type', 'ebook');

        $booksSoldInDept = $requestedDept->getBooksSoldInDept();


        // $allRelatedClients = $this->getClientsBasedOnOrderType($request, 'hardcover');
        return view(
            'snit.auth.dashboard.clients.clients_by_book.hardcover_clients',
            compact('allRelatedClients', 'table_headers', 'requestedDept', 'booksSoldInDept', 'hideFromPrint')
        );
    }


    /**
     * Not in use anymore
     * 
     * The page that shows a list of all the clients related to a specific book that ordered
     * hardcover books only
     * this way we can see all those who have paid or bought a specific book
     *
     */
    public function book_clients_ebook_show(Request $request, $deptSlug)
    {
        $table_headers = $this->getTableHeaders();
        $requestedBook = $request['requestedBook'] = Book::whereSlug($deptSlug)->first();
        $searchfor = $request->searchfor ?? '';

        $allRelatedClients = $this->getClientsBasedOnOrderType($request, 'ebook');
        return view(
            'snit.auth.dashboard.clients.clients_by_book.ebook_clients',
            compact('allRelatedClients', 'table_headers', 'requestedBook', 'searchfor')
        );
    }


    /**
     * gets clients for a specific book based on (order)type i.e ebook, hardcover or both
     *
     * @param Request $request
     * @param string $bookSlug
     * @param string|null $type refers to the order type: ebook or hardcover. 
     * 
     * if $type is not provided, then both ebook and hardcover order clients are returned.
     * 
     * @return \Illuminate\Support\Collection
     */
    public function getClientsBasedOnOrderType(Request $request, string $type = null): Collection
    {
        $allClients = [];

        // use slug to find book
        $requestedDept = $request['requestedDept'];

        // we ain't using unique clients here, wanna see it all.
        foreach ($requestedDept->service->getClients($type) as $key => $client)
            $allClients[$key] = $client;

        return $allRelatedClients = $this->filteredResults($request, collect($allClients)) ?? collect($allClients);

    }

    public function getTableHeaders()
    {
        return $table_headers = [
            'Org',
            'Client Name',
            'Mat. Number',
            // "isbn",
            'Client <br>Phone',
            'Purchase <br> Date',
            'Book <br> Purchased',
            'Purchase <br> Type',
            'income(₦)',
            "rec'd <br> sign",
            // 'ebook <br>income(₦)',
        ];
    }

    public function getHeadersToHideFromPrint()
    {
        return ['income(₦)', 'Mat. Number', 'Organisation', 'Book <br> Purchased', 'Client <br>Phone'];
    }
}
