<?php

namespace App\Http\Controllers;

use App\Http\Requests\StoreDocumentRequest;
use App\Http\Requests\UpdateDocumentRequest;
use App\Models\Book;
use App\Models\Document;
use Illuminate\Contracts\View\View;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use Intervention\Image\Facades\Image;
use stdClass;

class DocumentController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        //
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create(Request $request): View
    {
        // $all_books = Book::orderBy('id', 'desc')->get();
        // dd('daniel_mar', $all_books);
        return ($request->uploadfiles && $theBook = Book::find($request->book)) ?
            view('snit.auth.dashboard.books.docs.create', compact('theBook')) :
            view('snit.auth.dashboard.books.docs.create', compact('all_books'));
    }

    public function createManually(Request $request)
    {
        $all_books = Book::orderBy('id', 'desc')->get();
        // dd('daniel_mar', $all_books, $request->uploadfiles);
        $theBook = (Book::find($request->book)) ?? Book::first();

        return view('snit.auth.dashboard.books.docs.create-manually', compact('all_books', 'theBook'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response| RedirectResponse
     */
    public function store(StoreDocumentRequest $request): RedirectResponse
    {
        // get the book and use the relationship to create the new document record
        $request['book'] = Book::find($request->book_id);
        if (!$request['book']) {
            return back()->with('warning', 'Invalid Book or Title');
        }
        // dd($request['book']->doc, $request['book'], );

        // $filePaths = $this->storeFilesToServer($request);

        try {
            // storing unique files in the server
            $filePaths = $this->storeFilesToServer($request);
            $this->createUniqueDocs($request, $filePaths);

            return $this->redirectWithCreateSms();
        } catch (\Exception $e) {
            return redirect()->back()->with('error', 'Something went wrong while uploading file!' . $e->getMessage());
        }
    }

    /**
     * Display the specified resource
     *
     * @return \Illuminate\Http\Response
     */
    public function show(Document $document)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Models\Document  $document
     * @return \Illuminate\Http\Response
     */
    public function edit(Request $request, Document $doc)
    {
        // $all_books = Book::orderBy('id', 'desc')->get();//unnecessary data
        $theBook = $doc->book;

        return view('snit.auth.dashboard.books.docs.edit', compact('theBook'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Document  $document
     * @return \Illuminate\Http\Response
     */
    public function update(UpdateDocumentRequest $request, Document $doc)
    {
        // dd($request->all(), $doc);
        // get the associated book
        if (!$request['book'] = Book::find($request->book_id)) {
            return back()->with('warning', 'Invalid Book Title');
        }

        try {
            $filePaths = $this->updateServerFiles($request); // storing unique files in the server
            $this->updateExistingDocs($request, $filePaths);

            return $this->redirectWithUpdateSms();
        } catch (\Exception $e) {
            return redirect()->back()->with('error', 'Something goes wrong while Updating file!' . $e->getMessage());
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @return \Illuminate\Http\Response
     */
    public function destroy(Document $document)
    {
        //
    }

    // helpers

    public function redirectWithCreateSms()
    {
        return redirect()->route('books.index')->with('success', 'File Uploaded Successfully!!');
    }

    public function redirectWithUpdateSms()
    {
        return redirect()->route('books.index')->with('success', 'File Updated Successfully!!');
    }

    public function storeMainDocumentToServer($request)
    {
        // main book document
        // if (isset($request->book_file)) {
        //    $bookFileName = now()->timestamp . $request->book_file->getClientOriginalName();
        //    return  $bookFilePath = $request->file('book_file')->storeAs('files/ebooks', $bookFileName, 'public');
        // }

        if (isset($request->book_file_url)) {
            return $bookFilePath = $request->book_file_url;
        }
    }

    public function storeFrontCoverImageToServer($request)
    {
        if ($request->front_cover) {
            $backCoverFile = $this->compressFile($request->file('front_cover'));
            $backCoverFilePath = "files/front_covers/$backCoverFile->name"; // generate file path in server

            return (Storage::disk('public')->put($backCoverFilePath, $backCoverFile->image)) ? $backCoverFilePath : null;
        }
    }

    public function storeBackCoverImageToServer($request)
    {
        if ($request->back_cover) {
            $backCoverFile = $this->compressFile($request->file('back_cover'));
            $backCoverFilePath = "files/back_covers/$backCoverFile->name"; //generate file path in server

            return (Storage::disk('public')->put($backCoverFilePath, $backCoverFile->image)) ? $backCoverFilePath : null;
        }
    }

    /**
     * This compresses images using Intervention Image Library
     */
    public function compressFile($requestFile)
    {
        // get filepath without extension
        $fileNameWithExtension = now()->timestamp . $requestFile->getClientOriginalName();

        $ext = pathinfo($fileNameWithExtension, PATHINFO_EXTENSION);
        $backCoverFileName = basename($fileNameWithExtension, '.' . $ext); // filename without ext
        $backCoverFileName .= '.png'; // save file with the .png extension

        // resize and format as png
        $image = Image::make($requestFile)->fit(300, 215, function ($constraint) {
            $constraint->upsize();
        })->encode('png', 80);

        $compressedFile = new stdClass;
        $compressedFile->name = $backCoverFileName;
        $compressedFile->image = $image;

        return $compressedFile;
    }

    public function storeFilesToServer($request)
    {
        $this->handleDuplicates($request['book']->doc);

        $bookFilePath = $this->storeMainDocumentToServer($request);

        $frontCoverFilePath = $this->storeFrontCoverImageToServer($request);

        // dd('fne');
        $backCoverFilePath = $this->storeBackCoverImageToServer($request);
        $filesArray = [
            'book_file' => $bookFilePath,
            'front_cover' => $frontCoverFilePath ?? null,
            'back_cover' => $backCoverFilePath ?? null,
        ];

        return $useful = array_filter($filesArray, fn($el) => (bool) $el !== false);
    }

    /**
     * Returns an array of data to be sent to the Model's update()
     */
    public function updateServerFiles($request): array
    {
        // dd($request->all(), $request->book->doc);

        // handle duplicates
        if ($request->book->doc) {
            $exfiles = $this->getDuplicateDocFiles($request->book->doc);
            $this->deleteFilesFromServer($exfiles); // now delete all existing files
        }

        // dd($request->only(['front_cover', 'back_cover', 'book_file']));
        $bookFilePath = $this->storeMainDocumentToServer($request);

        $frontCoverFilePath = $this->storeFrontCoverImageToServer($request);

        $backCoverFilePath = $this->storeBackCoverImageToServer($request);

        $filesArray = [
            'book_file' => $bookFilePath ?? '',
            'front_cover' => $frontCoverFilePath ?? null,
            'back_cover' => $backCoverFilePath ?? null,
        ];

        return $useful = array_filter($filesArray, fn($el) => (bool) $el !== false);
    }

    /**
     * handles duplicate files stored on the server
     */
    public function handleDuplicates($duplicate)
    {
        if ($duplicate) {
            $exfiles = $this->getDuplicateDocFiles($duplicate);
            $this->deleteFilesFromServer($exfiles); // now delete all existing files
            $this->deleteRecords($duplicate); // now delete all existing db records
        }
    }

    public function deleteRecords($duplicate)
    {
        try {
            $duplicate->delete();
        } catch (\Exception $ex) {
            return redirect()->back()->with('error', 'Oops! Error removing duplicate records');
        }
    }

    public function getDuplicateDocFiles(Document $duplicate)
    {
        // if there are duplicate, you want to ideally replace the files and not add/create new files
        $exfilesArray = [
            'exFilePath' => formatPath($duplicate->book_file),
            //uses google url
            'exFrontCover' => formatPath($duplicate->front_cover),
            'exBackCover' => formatPath($duplicate->back_cover),
        ];

        return $exfiles = array_filter($exfilesArray, fn($el) => (bool) $el !== false);
    }

    public function deleteFilesFromServer(array $files)
    {
        try {
            foreach ($files as $file) {
                Storage::disk('public')->exists($file) ? Storage::disk('public')->delete($file) : null;
            }
        } catch (\Exception $e) {
            return back()->with('error', 'Files could not be Overwritten');
        }
    }

    /**
     * creates unique documents in the database
     */
    public function createUniqueDocs($request, $filePaths)
    {
        if (empty($request['book']->doc)) {
            try {
                $request['book']->doc()->create($filePaths);
            } catch (\Exception $e) {
                return back()->with('error', 'Oop! Document could not be created. Try again later');
            }
        }
    }

    /**
     * updates existing unique documents in the database
     */
    public function updateExistingDocs($request, $filePaths)
    {
        //we only update doc that already exists
        try {
            $request['book']->doc?->update($filePaths);
        } catch (\Exception $e) {
            return back()->with('error', 'Oop! Document could not be updated. Try again later');
        }
    }
}
