<?php

namespace App\Http\Controllers;

use App\Models\Author;
use App\Models\Book;
use Gloudemans\Shoppingcart\Contracts\Buyable;
use Gloudemans\Shoppingcart\Facades\Cart;
use Illuminate\Contracts\View\View;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;

class GuestCartController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response | View
     */
    public function index()
    {

        // print_r(Cart::destroy(), Cart::content()->count());
        $cartItemsIDs = [];
        $table_headers = [
            'front cover',
            'Book Title',
            'Book Author',
            'Organisation',
            // "ISBN",
            'Qty',
            'Type',
            'Price (₦)',
            'Actions'
            // "<i class='bx bx-trash text-danger'>",
        ];

        return view('snit.guest.cart.index', compact('table_headers'));
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request, Book $product, Author $author): Response|RedirectResponse
    {
        // if ($product instanceof Buyable)// dd($product); //Cart::add($product) // I'll look into why this does not work
        // avoid duplicates
        if (getCartItemByID($product->id)) {
            return back()->with('warning', 'You cannot choose ebook and hardcover for one product');
        }
        $item = [
            'id' => $product->id,
            'name' => $product->name,
            'qty' => 1,
        ];
        if ($request->type == 'ebook') {
            $item['price'] = $product->presentEbookPrice(); //for buyers
            $item['options'] = ['book_author' => $author, 'type' => 'ebook', 'original_price' => $product->ebookPrice()];
        } else {
            $item['price'] = $product->presentPrice();
            $item['options'] = ['book_author' => $author, 'type' => 'hardcover', 'original_price' => $product->price];
        }

        $newCartItem = Cart::add($item)->associate(Book::class);

        if ($newCartItem) {
            return back()->with('success', 'Item Added to Cart');
        }

        return back()->with('warning', 'Error Adding to Cart');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\RedirectResponse
     */
    public function destroy(Request $request)
    {
        if ($request->item_id == null) {
            Cart::destroy();
            // dd(Cart::content());
            return back()->with('info', 'null values provided');
        } else {
            // find the rowId using the item_id since the item_id is unique for each item in the cart
            $foundCartItem = Cart::search(function ($cartItem, $rowId) use ($request) {
                return (int) $cartItem->id == (int) $request->item_id;
            });
            Cart::remove($foundCartItem->first()->rowId);

            return back()->with('success', 'Item removed from cart');
        }
    }

    // hehlpers
    public function checkout()
    {
        dd('Checkout');
    }
}
