<?php

namespace App\Http\Controllers;

use stdClass;
use App\Models\Book;
use App\Models\User;
use App\Models\Order;
use App\Models\Author;
use App\Models\Client;
use Illuminate\Http\Request;
use App\Mail\ReplyContactFormEmail;
use Illuminate\Support\Facades\Mail;
use Gloudemans\Shoppingcart\Facades\Cart;
use App\Mail\InformDeveloperOfNewContactFormEntry;

class GuestController extends Controller
{
    /**
     * view store for all authors and all institutions.
     * all books uploaded by all authors in all institutions can be accessed here
     */
    public function bookstore()
    {
        $allbooks = Book::with('author')->get();

        $cartItemsIDs = [];

        foreach (Cart::content() as $cartItem) {
            $cartItemsIDs[$cartItem->id] = $cartItem;
        }

        return view('snit.auth.dashboard.store.general_store', compact('allbooks', 'cartItemsIDs'));
    }

    /**
     * view store for a particular author
     */
    public function viewstore($author)
    {
        $relatedUser = User::whereName($author)->first();
        $author = $relatedUser->author;
        // foreach (Cart::content() as $cartItem) $cartItemsIDs[] = $cartItem->id;
        foreach (Cart::content() as $cartItem) {
            $cartItemsIDs[$cartItem->id] = $cartItem;
        }
        /**
         * @var $cartItemIDs represents the list of book ids that are in the cart
         */
        $cartItemsIDs = $cartItemsIDs ?? [];
        $allbooks = $author->books;

        return view('snit.auth.dashboard.store.index_v2', compact('allbooks', 'author', 'cartItemsIDs'));
    }

    public function viewvendor(Author $author)
    {
        dd('profile of vendor ', $author);
    }

    public function searchResults(Request $request)
    {
        $searchfor = $request->searchfor;
        $cartItemsIDs = $this->getCartItemsWithKeys();

        if ($searchfor == 'store')
            return back()->with('error', "invalid search term : `$searchfor`");

        if (strstr($searchfor, '/')) {
            // if string contains a '/' it's most likely a mat_num
            // perform client search based on mat_num
            $foundClient = Client::where('mat_num', $searchfor)?->first();

            if (!blank($foundClient) and $request['mat_num'] = $searchfor)
                return $this->showAllClientOrders($request);
        }

        $relatedAuthors = $this->searchForRelatedAuthorsForClientRequest($searchfor);

        $relatedBooks = $this->searchForRelatedBooksForClientRequest($searchfor);

        // perform Order ID search if no authors or books are found
        if (blank($relatedAuthors) and blank($relatedBooks)) {
            return redirect()->route('guest.search_for_my_order', $searchfor);
        }
        // else: return authors and books found
        return view(
            'snit.guest.search.results',
            compact('relatedAuthors', 'relatedBooks', 'cartItemsIDs', 'searchfor')
        );
    }





    /**
     * Get items in cart as an assoc array such that the keys refer to the actual cartItem keys and the value holds the item (info)
     * 
     * key : cartItemkey in the cart
     * 
     * value: product information in the cart
     */
    public function getCartItemsWithKeys()
    {
        // for checking if an item has already been added to the cart
        foreach (Cart::content() as $cartItem) {
            $cartItemsIDs[$cartItem->id] = $cartItem;
        }

        return $cartItemsIDs = $cartItemsIDs ?? [];
    }
    public function searchForRelatedAuthorsForClientRequest($searchfor)
    {
        /**
         * @var string $like represents the case-insensitive 'like' operator for sql query
         */
        $like = (env('DB_CONNECTION') == 'mysql') ? 'like' : 'ilike'; //mariadb vs pgsql

        $relatedUserIDs = User::where('name', "$like", "%$searchfor%")->pluck('id');

        $relatedAuthorsUserIDs = Author::where('fname', "$like", "%$searchfor%")
            ->orWhere('lname', "$like", "%$searchfor%")->pluck('user_id');

        $userIDsCombo = array_merge($relatedAuthorsUserIDs->toArray(), $relatedUserIDs->toArray());

        $uniqueUserIDs = array_unique($userIDsCombo);

        $relatedAuthors = []; //create an array to hold all the related authors objects

        foreach ($uniqueUserIDs as $id) {
            $relatedAuthors[] = Author::where('user_id', $id)->first();
        }

        return $relatedAuthors;
    }

    public function searchForRelatedBooksForClientRequest($searchfor)
    {
        /**
         * @var string $like represents the case-insensitive 'like' operator for sql query
         */
        $like = (env('DB_CONNECTION') == 'mysql') ? 'like' : 'ilike'; //mariadb vs pgsql

        // this returns only related books that have documents linked to them.
        return $relatedBooks = Book::query()->has('doc')->with('doc')->where('name', "$like", "%$searchfor%")->get();

    }
    public function findMyOrder()
    {
        $searchfor = request()->order_id;

        if ($relatedOrder = Order::successful()->where('identifier', $searchfor)->first()) {
            $relatedContent = json_decode($relatedOrder->content, 1);
            $ordered_content_models = array_map(fn($item) => Book::find($item['id']), $relatedContent);
            return view('snit.guest.search.order', compact('searchfor', 'ordered_content_models'));
        }
        if ($relatedOrder = Order::where('identifier', $searchfor)->first()) {
            return back()->with('error', "Order $searchfor is " . strtoupper($relatedOrder->status));
        }
    }

    public function showAllClientOrders(Request $request)
    {
        if (!$request->mat_num)
            return back()->with('error', 'No mat Number provided');

        $table_headers = $this->getTableHeaders();

        $foundClient = Client::where('mat_num', $request->mat_num)->first();

        $relatedOrders = $foundClient?->orders;

        if (!$foundClient)
            return back()->with('error', 'Invalid matric number');
        // dd($relatedOrders);
        /**
         * @var array $allOrders stores the array of all the unique data for each order made by this matric. num
         */
        $allOrderedItems = [];
        foreach ($relatedOrders as $clientOrder) {
            // dd($key, $orderName);
            $orderBasket = $clientOrder->getBasket();
            foreach ($orderBasket as $orderItem) {
                // dd($orderItem);
                $allOrderedItems[] = [
                    'name' => $orderItem->name,
                    'price' => $orderItem->price,
                    'model' => Book::with('author')->find($orderItem->model->id),
                    'type' => $orderItem->options->type,
                    'datePurchased' => $clientOrder->getPurchaseDate(),
                ];
            }
        }
        $searchfor = $request->mat_num ?? '';
        return view(
            'snit.guest.search.client_history',
            compact('foundClient', 'allOrderedItems', 'table_headers', 'searchfor')
        );
    }

    public function findMyOrderResults()
    {
        dd('return results for my order page');
    }




    public function getTableHeaders()
    {
        return $table_headers = [
            'Client Name',
            'Mat. Number',
            'Client Phone',
            'Book <br> Purchased',
            'price(₦)',
            'Type',
            'author',
            'date <br> purchased',
        ];
    }


    // contact developer

    public function contact_developer(Request $request)
    {
        // organise user input
        $client = new stdClass;
        $client->fullname = $request['form-name'];
        $client->email = $request['form-email'];
        $client->subject = $request['form-subject'];
        $client->message = $request['message'];

        // dd($client);
        try {
            $this->sendAcknowledgmentMailToGuest($client);
            $this->sendEmailNotificationToDeveloper($client);
        } catch (\Throwable $th) {
            return 'Oops! Error occured. <br> Reason : ' . $th->getMessage();
        }
        dd('Emails sent successfully');
    }

    public function sendAcknowledgmentMailToGuest($client)
    {
        Mail::to($client->email)->send(new ReplyContactFormEmail($client));
    }

    public function sendEmailNotificationToDeveloper($client)
    {
        Mail::to(env('DEVELOPER_MAIL', 'updatedaniels@gmail.com'))
            ->send(new InformDeveloperOfNewContactFormEntry($client));
    }
}
