<?php

namespace App\Http\Controllers;

use App\Http\Requests\ProfileUpdateRequest;
use App\Models\Organisation;
use App\Traits\Banks\NigerianBanksTrait;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Redirect;
use Illuminate\View\View;

class ProfileController extends Controller
{
    use NigerianBanksTrait;
    /**
     * Display the user's profile form.
     */
    public function edit(Request $request): View
    {
        $orgs = Organisation::get(['id', 'name']);
        // dd(authy()->id);
        return view('snit.auth.account-settings', [
            'user' => $request->user(),
            'orgs' => $orgs,
            'naija_banks' => $this->getSupportedNigerianBanks()
        ]);
    }

    /**
     * Update the user's profile information.
     */
    public function update(ProfileUpdateRequest $request): RedirectResponse
    {
        // research user->fill() vs user->update()

        // $request->user()->fill($request->validated()); ??it seems fill does not update,
        // it just assigns new values to the model (in memory) without making the db query
        $request->user()->update($request->validated());

        $validatedColl = collect($request->validated());

        // if the related Author info was  (passed through) changed, then update the author details in db
        $validated2 = $validatedColl->except(['name', 'email'])->toArray();
        $request->user()->author->update($validated2);

        // if the user email was changed, then reset the email verified column
        if ($request->user()->isDirty('email')) {
            $request->user()->email_verified_at = null;
        }

        // save all changes to user and related models
        $request->user()->push();

        return Redirect::route('profile.edit')->with('success', 'profile-updated');
    }

    /**
     * Delete the user's account.
     */
    public function destroy(Request $request): RedirectResponse
    {
        $request->validateWithBag('userDeletion', [
            'password' => ['required', 'current-password'],
        ]);

        $user = $request->user();

        Auth::logout();

        $user->delete();

        $request->session()->invalidate();
        $request->session()->regenerateToken();

        return Redirect::to('/');
    }
}
