<?php

namespace App\Http\Controllers;

use App\Models\Dept;
use Illuminate\Http\Request;
use Illuminate\Contracts\View\View;

class ReportGeneratorController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        //
    }

    /**
     * Show the form for creating a new resource.
     * Show form to Generate a report for a specific dept
     *
     * @return \Illuminate\Http\Response | View
     */
    public function create(Request $request): \Illuminate\Http\Response|View
    {
        if ($request->dept) {
            // return $this->presentReport();
            return redirect()->route('user.show.dept-report', ['dept' => $request->dept]);
        }

        return view('snit.auth.dashboard.depts.reports_create', [
            'dept_levels' => $this->dept_levels,
        ]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response | \Illuminate\Http\RedirectResponse | View
     */
    public function show($id): \Illuminate\Http\Response|\Illuminate\Http\RedirectResponse|View
    {
        $dept = Dept::findOrFail(request()->dept);

        // authorize : refactor code using policies
        if ($dept->user_id !== auth()->user()->id)
            return back()->with('failed', 'You are not authorized');

        $allClients = $finalClients = []; // get All clients for this dept

        $authorBooks = author()->books;

        foreach ($authorBooks as $currentBook) // one liner
            foreach ($currentBook->service->getClients() as $key => $client)
                $allClients[$key] = $client; // one liner

        foreach ($allClients as $key => $client)
            if (matchMatNum($client, $dept))
                $finalClients[$key] = $client; //append to final list.

        $allClients = $finalClients; //very important

        $table_headers = [
            'client_name',
            'dept <br> level',
            'matric_num',
            'book <br> purchased',
            'sample <br> mat num',
            'department'
        ];

        return view('snit.auth.dashboard.depts.show', compact('allClients', 'table_headers'), [
            'dept_levels' => $this->dept_levels,
            'dept' => $dept,
        ]);
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }


    // helper

    public function downloadReport(Request $request)
    {
        dd('download dept record for : ' . $request->dept);
    }

}
