<?php

namespace App\Http\Requests;

use App\Models\Organisation;
use Illuminate\Validation\Validator;
use Illuminate\Foundation\Http\FormRequest;

class GuestCheckoutRequest extends FormRequest
{

    public bool $existingClient = false;
    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {
        return true;
    }



    public function prepareForValidation()
    {
        $emptyNames = is_null($this->firstName) and is_null($this->lastName);
        $emptyOrganisation = is_null($this->organisation);
        // determine is new or existing customer
        if ($emptyNames and $emptyOrganisation) 
            $this->existingClient = true;
        else {
            $org = Organisation::where('name', $this->organisation)->first();
            $this->merge(['sch' => $org?->name]);
            $this->merge(['org_id' => $org?->id]);
        }
        // unmerged ['email',address, state, mat_num]
    
        $this->merge([
            'fname' => $this->firstName,
            'lname' => $this->lastName,
            'phone' => str_replace(' ', '', $this->phoneNumber),
            'country_id' => $this->country,
            'mat_num' => trim(str_replace(' ', '', $this->mat_num)),
        ]);

        if (empty($this->mat_num)) {
            $this->merge(['is_student' => false]);
        }
    }


    /**
     * Get the validation rules that apply to the request.
     *
     * @return array<string, mixed>
     */
    public function rules(): array
    {
        return $this->existingClient ? $this->validateOnlyMatNum() : $this->myRules();
    }

    public function validateOnlyMatNum()
    {
        return [
            'mat_num' => 'required|exists:clients'
        ];
    }

    public function myRules(): array
    {
        return [
            // client data
            'is_student' => 'nullable',
            'fname' => 'required|string|min:5|max:20',
            'lname' => 'required|string|min:5|max:20',
            'phone' => 'required|string|min:11|max:15', //unique:clients,phone
            'email' => 'required', //unique:clients,email

            // for student clients
            'state' => 'nullable|string',
            'country_id' => 'nullable|integer',
            'sch' => 'required|string',
            'org_id' => 'required|int',
            'address' => 'nullable|string|max:50',
            'mat_num' => 'nullable|string|max:20',
        ];
    }
    public function messages(): array
    {
        return
            [
                'fname.required' => 'Your First Name field is required',
                'lname.required' => 'Your Last Name field is required',
                'fname.min' => 'Your First Name must be at least 5 characters.',
                'fname.max' => 'Your first name must not be greater than 20 characters.',
                'lname.min' => 'Your Last Name must be at least 5 characters.',
                'lname.max' => 'Your Last name must not be greater than 20 characters.',
            ];
    }
}
