<?php

namespace App\Http\Requests;

use App\Models\User;
use App\Models\Organisation;
use Illuminate\Validation\Rule;
use Illuminate\Foundation\Http\FormRequest;

class ProfileUpdateRequest extends FormRequest
{
    /**
     * Get the validation rules that apply to the request.
     *
     * @return array<string, \Illuminate\Contracts\Validation\Rule|array|string>
     */
    public function rules(): array
    {
        return [
            'name' => ['string', 'max:255'],
            'email' => ['email', 'max:255', Rule::unique(User::class)->ignore($this->user()->id)],
            'fname' => 'sometimes|max:50',
            'lname' => 'sometimes|max:50',
            'phone' => 'sometimes|max:15',
            'org_id' => 'sometimes|int',
            'address' => 'sometimes|max:50',
            'state' => 'sometimes|max:20',
        ];
    }

    public function prepareForValidation()
    {
        foreach ($this->all() as $key => $value) {
            if (strtolower($value) == 'not set') {
                unset($this[$key]);
            }
        }

        // conforming to db standards (column names)
        return $this->merge([
            'name' => $this->username,
            'fname' => $this->firstName,
            'lname' => $this->lastName,
            'org_id' => (int)Organisation::whereName($this->organisation)->first()->id,
            'phone' => $this->phoneNumber,
        ]);

    }
}
