<?php

namespace App\Http\Requests;

use App\Models\Publisher;
use Illuminate\Foundation\Http\FormRequest;

class StoreBookRequest extends FormRequest
{
    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {
        return true;
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array<string, mixed>
     */
    public function rules()
    {
        return [
            'name' => 'required',
            'ISBN' => 'required',
            'price' => 'required',
            'publisher' => 'required_without:publisher_id',
            'publisher_id' => 'required_without:publisher',
            'pub_year' => 'required|integer',
            // nullable fields
            'inventory' => 'sometimes|min:0',
            'message' => 'nullable',
            // 'type' => 'sometimes',
        ];
    }

    public function messages()
    {
        return [
            'publisher.*' => 'publisher information is required',
            'publisher_id' => 'publisher information is required',
        ];
    }

    public function prepareForValidation()
    {
        $this->organisePublisherData(); // organise the publisher data
    }

    public function organisePublisherData()
    {
        if ($this->publisher && $found = Publisher::whereCoyName($this->publisher)->first()) { // if an already existing publisher was chosen
            $this->merge(['publisher_id' => $found->id]);
        } // set publihser id
        // this code block will only execute if a new publisher was indicated in the form
        elseif ($this->publisher && ($newPublisher = Publisher::create(['coy_name' => $this->publisher]))) {
            $this->merge(['publisher_id' => $newPublisher->id]);
        } // set publihser id
        // $this->merge(['name' => $this->title]); //default
    }
}
