<?php

namespace App\Mail;

use App\Models\Client;
use Illuminate\Bus\Queueable;
use Illuminate\Mail\Mailable;
use Illuminate\Mail\Mailables\Address;
use Illuminate\Mail\Mailables\Content;
use Illuminate\Mail\Mailables\Envelope;
use Illuminate\Queue\SerializesModels;

class OrderConfirmationMail extends Mailable
{
    use Queueable, SerializesModels;

    public $relatedOrder;

    public $relatedClient;

    public $relatedContent;

    /**
     * Create a new message instance.
     *
     * @return void
     */
    public function __construct($related_order)
    {
        $this->relatedOrder = $related_order;
        $this->relatedClient = Client::find($this->relatedOrder->client_id);

        $raw_relatedContent = json_decode($this->relatedOrder->content);
        $this->relatedContent = collect($raw_relatedContent)->toArray();
    }

    /**
     * Get the message envelope.
     *
     * @return \Illuminate\Mail\Mailables\Envelope
     */
    public function envelope()
    {
        return new Envelope(
            from: new Address('support@aims.marxon.com.ng', 'AIMS'),
            subject: 'New Order Confirmation',
        );
    }

    /**
     * Get the message content definition.
     *
     * @return \Illuminate\Mail\Mailables\Content
     */
    public function content()
    {
        return new Content(
            markdown: 'mail.order-confirmation-mail',
        );
    }

    /**
     * Get the attachments for the message.
     *
     * @return array
     */
    public function attachments()
    {
        return [];
    }
}
