<?php

namespace App\Mail;

use App\Models\Client;
use Illuminate\Bus\Queueable;
use Illuminate\Mail\Mailable;
use App\PaymentGatewayServices\Rexpay;
use Illuminate\Mail\Mailables\Content;
use Illuminate\Queue\SerializesModels;
use Illuminate\Mail\Mailables\Envelope;
use Illuminate\Contracts\Queue\ShouldQueue;

class OrderVerificationFailedWithPGSMail extends Mailable
{
    use Queueable, SerializesModels;


    public $relatedOrder;

    public $relatedClient;

    public $relatedContent;

    public $pgsData;


    /**
     * Create a new message instance.
     *
     * @return void
     */
    public function __construct($related_order)
    {
        $this->relatedOrder = $related_order;
        $this->relatedClient = Client::find($this->relatedOrder->client_id);

        $raw_relatedContent = json_decode($this->relatedOrder->content);

        $this->relatedContent = collect($raw_relatedContent)->toArray();
        
        $this->fetchOrderInfoFromPGS();
    }
    /**
     * Get the message envelope.
     *
     * @return \Illuminate\Mail\Mailables\Envelope
     */
    public function envelope()
    {
        return new Envelope(
            subject: 'Alert! A successful Order Verification Failed With PGS',
        );
    }

    /**
     * Get the message content definition.
     *
     * @return \Illuminate\Mail\Mailables\Content
     */
    public function content()
    {
        return new Content(
            markdown: 'mail.order-verification-failed-with-p-g-s-mail',
        );
    }

    /**
     * Get the attachments for the message.
     *
     * @return array
     */
    public function attachments()
    {
        return [];
    }


    public function fetchOrderInfoFromPGS()
    {
        /**
         * refers to the payment gateway used in the transaction
         */
        $pgsUsedInOrder = $this->relatedOrder->pgs;
        $pgs = $pgsUsedInOrder->name;

        if ($pgs == 'rexpay') 
            $this->pgsData = $this->handleRexpay();
    }

    private function handleRexpay(): array
    {
        $res = (new Rexpay)->verifyPaymentRequest($this->relatedOrder);
            $resBody = json_decode($res->body(), 1);
            $resBody['http_code'] = $res->getStatusCode();
            return $resBody;
    }
}
