<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class Author extends Model
{
    use HasFactory;

    protected $guarded = ['id', 'org_id'];

    //  Relationships

    public function books()
    {
        return $this->hasMany(Book::class);
    }

    public function user()
    {
        return $this->belongsTo(User::class);
    }

    public function org()
    {
        return $this->belongsTo(Organisation::class, 'org_id', 'id');
    }

    // helpers

    // helpers: getters

    // for authors

    public function totalSalesFromBooks($duration = null): float
    {
        $total_sales = 0.00;

        foreach ($this->books as $book)
            $total_sales += $book->presentTotalSalesGenerated($duration);
        return $total_sales;
    }

    public function totalBookSold($duration = null)
    {
        try {

            switch ($duration) {

                case 'today':
                    return $this->totalBookSoldToday();

                case 'this_week':
                    return $this->totalBookSoldThisWeek();

                case 'this_month':
                    return $this->totalBookSoldThisMonth();

                case 'this_year':
                    return $this->totalBookSoldThisYear();

                case 'yesterday':
                    return $this->totalBookSoldYesterday();

                case 'last_week':
                    return $this->totalBookSoldLastWeek();

                case 'last_month':
                    return $this->totalBookSoldLastMonth();

                case 'last_year':
                    return $this->totalBookSoldLastYear();

                case null:
                case '':
                    return $this->totalBookSoldAllTime();

                default:
                    return throw new \Exception("Error Processing Request : '$duration' is not specified in your match code block", 1);
            }
        } catch (\Throwable $th) {
            dd($th->getMessage());
        }
    }

    public function totalBookSoldToday(): int
    {
        /**
         * we do this by looping each book and adding all the books sold in the current month altogether.
         */
        $totalBookSoldToday = 0.00;
        foreach ($this->books as $book) {
            $totalBookSoldToday += $book->service->getTotalNumberSold('today');
        }

        return (int) $totalBookSoldToday;
    }

    public function totalBookSoldYesterday(): int
    {
        /**
         * we do this by looping each book and adding all the books sold in the current month altogether.
         */
        $totalBookSoldYesterday = 0.00;
        foreach ($this->books as $book) {
            $totalBookSoldYesterday += $book->service->getTotalNumberSold('yesterday');
        }

        return (int) $totalBookSoldYesterday;
    }

    public function totalBookSoldThisWeek(): int
    {
        /**
         * we do this by looping each book and adding all the books sold in the current month altogether.
         */
        $totalBookSoldThisWeek = 0.00;
        foreach ($this->books as $book) {
            $totalBookSoldThisWeek += $book->service->getTotalNumberSold('this_week');
        }

        return (int) $totalBookSoldThisWeek;
    }

    public function totalBookSoldLastWeek(): int
    {
        /**
         * we do this by looping each book and adding all the books sold in the current month altogether.
         */
        $totalBookSoldLastWeek = 0.00;
        foreach ($this->books as $book) {
            $totalBookSoldLastWeek += $book->service->getTotalNumberSold('last_week');
        }

        return (int) $totalBookSoldLastWeek;
    }

    public function totalBookSoldThisMonth(): int
    {
        /**
         * we do this by looping each book and adding all the books sold in the current month altogether.
         */
        $totalBookSoldThisMonth = 0.00;
        foreach ($this->books as $book) {
            $totalBookSoldThisMonth += $book->service->getTotalNumberSold('this_month');
        }

        return (int) $totalBookSoldThisMonth;
    }

    public function totalBookSoldLastMonth(): int
    {
        /**
         * we do this by looping each book and adding all the books sold in the current month altogether.
         */
        $totalBookSoldLastMonth = 0.00;
        foreach ($this->books as $book) {
            $totalBookSoldLastMonth += $book->service->getTotalNumberSold('last_month');
        }

        return (int) $totalBookSoldLastMonth;
    }

    public function totalBookSoldThisYear(): int
    {
        /**
         * we do this by looping each book and adding all the books sold in the current month altogether.
         */
        $totalBookSoldThisYear = 0.00;
        foreach ($this->books as $book) {
            $totalBookSoldThisYear += $book->service->getTotalNumberSold('this_year');
        }

        return (int) $totalBookSoldThisYear;
    }

    public function totalBookSoldLastYear()
    {
        /**
         * we do this by looping each book and adding all the books sold in the current month altogether.
         */
        $totalBookSoldLastYear = 0.00;
        foreach ($this->books as $book) {
            $totalBookSoldLastYear += $book->service->getTotalNumberSold('last_year');
        }

        return (int) $totalBookSoldLastYear;
    }

    public function totalBookSoldAllTime(): int
    {
        $totalBookSold = 0.00;
        foreach ($this->books as $book) {
            $totalBookSold += $book->service->getTotalNumberSold();
        }

        return (int) $totalBookSold;
    }

    // for authors

        
    /**
     * getAllClients gets all the clients an author has as an assoc array in the format:
     * 
     * $allClients[$timeOfOrdering] = $client;
     *
     * @return array
     */
    public function getAllClients()
    {
        $allClients = [];
        foreach ($this->books as $currentBook) {
            foreach ($currentBook->service->getClients() as $timeOfOrdering => $client)
                $allClients[$timeOfOrdering] = $client;
        }
        krsort($allClients); //SORT ARRAY BY KEYS

        return $allClients;
        // dd($allClients);
    }

    /**
     * gets a column of data from clients eg all mat_nums, phone, (sch) etc
     */
    public function getAllClientInfo($prop = 'mat_num')
    {
        $arrayOfResults = [];

        foreach ($this->getAllClients() as $timeOfOrdering => $client)
            $arrayOfResults[] = $client->client_model->$prop;

        return $arrayOfResults;
    }



    // presenters


    public function presentWallet()
    {
        return number_format($this->wallet(), 2);
    }
    public function presentTotalProfit($duration = null): float
    {
        $hardCoverProfit = $this->presentHardCoverProfit($duration);

        $eBookProfit = $this->presentEbookProfit($duration);

        return $totalProfit = $eBookProfit + $hardCoverProfit; // compute gross profit
    }

    public function presentEbookProfit($duration = null): float
    {
        $eBookProfit = 0.00;
        // compute ebook profit : All sales = profits in eBook system
        foreach ($this->books as $book) {
            $eBookProfit += $book->service->presentTotalEbookSales($duration);
        }

        return (float) $eBookProfit;
    }

    /**
     * presents the hardcover sales profit made during a specific time frame for the author.
     *
     * time frames include today, this week, this month this year and for the past also.
     *
     *  like yesterday, last week, last month last year __not implemented
     *
     * When the duration is not provided, it returns hardcover profits(all time)
     */
    public function presentHardCoverProfit($duration = null): float
    {
        $hardCoverSales = 0.00;
        //To compute hardcover profit

        // step 1: add up all the sales first of all
        foreach ($this->books as $book) {
            $hardCoverSales += $book->service->presentTotalHardcoverSales($duration);
        }

        // step 2: compute the profit based on hardcopy
        return $hardCoverProfit = (float) $this->computeHardcoverProfit($hardCoverSales);
    }

    public function computeHardcoverProfit($hardCoverSales)
    {
        $hardCoverProfit = $hardCoverSales * profitMargin(); //divide the  total sales by (3 or 4) to get the average profit on each hardcover book sold
        $hardCoverProfit = round($hardCoverProfit / 10) * 10; // roundup to the nearest 10s to avoid amounts like N12 or N19

        // dd($hardCoverSales, $hardCoverProfit);

        return $hardCoverProfit = round($hardCoverProfit / 10) * 10; // roundup to the nearest 10s to avoid amounts like N12 or N19
    }

    public function getNationality($attr = null)
    {
        if ($foundNationality = Country::where('id', $this->country_id)?->get(['id', 'name'])->first()) {
            return $attr == 'id' ? $foundNationality->id : ucfirst($foundNationality->name);
        }

        return false;
    }

    public function getOrg($attr = null)
    {
        if ($foundOrg = Organisation::where('id', $this->org_id)?->get(['id', 'name', 'slug'])->first()) {
            return $attr ? $foundOrg->$attr : ucfirst($foundOrg->name);
        }

        return false;
    }

    public function presentBookCount()
    {
        $totalBooks = count($this->books);
        $presenter = match ($totalBooks) {
            1 => 'book',
            default => 'books',
        };

        return $totalBooks . ' ' . $presenter;
    }

    public function presentFullName()
    {
        return ucwords($this->fname . ' ' . $this->lname);
    }


    public function getBank(): BankAccount|null
    {
        return BankAccount::where('user_id', $this->user_id)->first();
    }

    // financials

    public function wallet()
    {
        return $this->totalSalesFromBooks('this_week');
    }

    public function changeInSales(string $durationFrom, string $durationTo) : float
    {
        return $this->totalSalesFromBooks($durationFrom) - $this->totalSalesFromBooks($durationTo);
    }

    public function changeInProfits(string $durationFrom, string $durationTo): float
    {
        return $this->presentTotalProfit($durationFrom) - $this->presentTotalProfit($durationTo);
    }
}
