<?php

namespace App\Models;

use App\Services\Book\BookService;
use App\Traits\ModelTraits\Book\BookAccessors;
use Illuminate\Database\Eloquent\Model;
use Gloudemans\Shoppingcart\CanBeBought;
use App\Traits\ModelTraits\Book\BookHelpers;
use Gloudemans\Shoppingcart\Contracts\Buyable;
use App\Traits\ModelTraits\Book\BookRelationships;
use Illuminate\Database\Eloquent\Factories\HasFactory;

class Book extends Model implements Buyable
{
   use HasFactory;
   use CanBeBought; // trait from Gloudemans Buyable interface

   /**
    * settings eg. table name ::stays in model
    * constructor ::stays in model
    * query scopes ::can be imported as trait
    */
   use BookRelationships; //relationships
   use BookHelpers; //minor dustbin :  uncategorized (Getters and setters)
   use BookAccessors; // Accessors and Mutators | Attributes

   protected $guarded = ['id'];

   /**
    * grants access to the service initialization for BookService
    *
    * @var BookService $service
    */
   public BookService $service;

   public function __construct()
   {
      parent::__construct();
      $this->service(); // register book service
   }
}
