<?php

namespace App\Models;

use App\Models\Client;
use Illuminate\Support\Collection;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Factories\HasFactory;

class Dept extends Model
{
    use HasFactory;

    protected $guarded = ['id'];

    public function user()
    {
        return $this->belongsTo(User::class);
    }

    // Presenters

    public function presentProgramType()
    {
        if ($this->program_type == 'ft') {
            return 'FULL TIME';
        }

        return 'PART TIME';
    }


    //helpers

    /**
     * getClients gets all clients from a specific department in the author's sch
     *
     * @return Collection
     */
    public function getClients(): Collection
    {
        // get the list of books linked to this dept by author()
        // dd($this->matric_num_fmt);
        // return Client::where('matric_num_fmt', "$this->matric_num_fmt")->get()->sortDesc();
        $theClients = collect(author()->getAllClients());
        // NB: this is no query builder it pure collections manipulation
        return $theClients
            ->where('client_model.matric_num_fmt', "$this->matric_num_fmt")
            ->where('client_model.org_id', "$this->org_id");
    }


    public function getBooksSoldInDept()
    {
        // return $booksSoldInDept = $this->books()->pluck('name');
        return $booksSoldInDept = $this->books()->get(['name', 'slug']);

    }

    // Relationships
    public function books()
    {
        return $this->belongsToMany(Book::class, 'book_dept');
    }
}
