<?php

namespace App\Models;

use App\Services\GoogleDriveService;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class Document extends Model
{
    use HasFactory;

    protected $guarded = ['id'];

    /**
     * use the following keys to use this api
     *  'bc' represents back_cover
     *  'fc' represents front_cover
     *  default is book
     */
    public function presentFileUrl($filetype = 'bk')
    {
        return match ($filetype) {
            'fc' => $this->getBase($this->front_cover),
            'bc' => $this->getBase($this->back_cover),
            default => $this->getBase($this->book_file),
        };
    }

    public function getBase($filepath, $delimiter = '_')
    {
        if ($filepath !== 'n/a') {
            $base = basename($filepath);
            $delimiter_pos = strpos($base, $delimiter);

            return substr($base, ++$delimiter_pos);
        }

        return $filepath;
    }

    //  relationship
    public function book()
    {
        return $this->belongsTo(Book::class);
    }

    // Google File Service
    public function getDirectLink(): string
    {
        return (new GoogleDriveService($this->book_file))->getDirectLink();
    }
}
