<?php

namespace App\Models;

use Illuminate\Support\Carbon;
use Illuminate\Support\Collection;
use App\Enums\Order\OrderStatusEnum;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Factories\HasFactory;

class Order extends Model
{
    use HasFactory;

    protected $guarded = ['id'];

    //  relationships
    public function client()
    {
        return $this->belongsTo(Client::class);
    }

    public function pgs()
    {
        return $this->belongsTo(PG::class, 'pgs_id', 'id');
    }


    // queryscopes

    /**
     * get all successful orders
     *
     * @param [type] $builder
     * @return Builder
     */
    public function scopeSuccessful($builder): Builder
    {
        return $builder->where('status', OrderStatusEnum::SUCCESS);
    }

    public function scopeAbove2400($builder): Builder
    {
        return $builder->where('total_amount', '>', '2400');
    }




    // helpers
    /**
     * represents all the items in a specific order, returned as a collection
     */
    public function getBasket(): Collection
    {
        return $basket = collect(json_decode($this->content));
    }


    /**
     * Returns the date the order was placed
     */
    public function getPurchaseDate(): string
    {
        return Carbon::createFromDate($this->created_at)->toDateString();
    }
}
