<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class Organisation extends Model
{
    use HasFactory;

    protected $guarded = ['id'];

    /**
     * this returns the IMS codes with their respective model id's as keys
     *eg. [1 => 'DSPG', 2 => 'DSPO']
     * @return array Array with a starting index of 1 (which is the lowest organisation id in the db)
     */
    public static function IMSCodes(): array
    {
        return Organisation::pluck('ims_code', 'id')->toArray();
    }

    public static function getRandomIMSCode(): string
    {
        return self::pluck('ims_code', 'id')->random();
    }
    /**
     * validates an IMS code
     *
     * @param string $imsCode
     * @return boolean
     */
    public static function validateIMSCode(string $imsCode): bool
    {
        return !in_array($imsCode, self::IMSCodes());
    }

    /**
     * this generates a unique ims_code for the org
     *
     * @return string
     */
    public function generateIMSCode(): string
    {
        return strtoupper($this->slug . '_' . $this->id);
    }

    public function getEndOfIMSCode()
    {
        return $endofCode = explode('_', $this->ims_code)[1];
    }

    /**
     * this ensures that all ims codes have a consistent format of  $slug_$id 
     * This also ensures that the code is unique in the db
     * @return bool
     */
    public function selfValidateIMSCode(): bool
    {
        $endofCode = $this->getEndOfIMSCode();
        if ($endofCode != $this->id) {
            $endofCode = $this->id;
            $this->ims_code = "{$this->slug}_{$this->id}";
            $this->save();
        }
        return $endofCode == $this->id;
    }
    public function users()
    {
        return $this->hasMany(User::class, 'org_id', 'id');
    }

    public function authors()
    {
        return $this->hasMany(Author::class, 'org_id');
    }

    public function orgAdmins()
    {
        return $this->hasMany(OrgAdmin::class);
    }

    public function clients()
    {
        return $this->hasMany(Client::class);
    }
}
