<?php

namespace App\Models;

// use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Illuminate\Support\Str;
use Laravel\Sanctum\HasApiTokens;

class User extends Authenticatable
{
    use HasApiTokens, HasFactory, Notifiable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [
        'name',
        'email',
        'password',
        'role',
        //users should not be allowed to update their org_id after account creation. only app admins are authorized
        // solution: call customer care for assistance
        'org_id',
        'is_admin',
    ];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array<int, string>
     */
    protected $hidden = [
        'password',
        'remember_token',
    ];

    /**
     * The attributes that should be cast.
     *
     * @var array<string, string>
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];

    //  relationships

    public function author()
    {
        return $this->hasOne(Author::class);
    }

    public function orgAdmin()
    {
        return $this->hasOne(OrgAdmin::class);
    }

    /**
     * returns a developer instance for the auth user; ie.if he is an admin or the dev
     *
     * @return void
     */
    public function dev()
    {
        return ($this->is_admin) ? new Developer : null;
    }

    public function depts()
    {
        return $this->hasMany(Dept::class);
    }

    public function relatedDepts(): Collection
    {
        return Dept::where('org_id', $this->org_id)->get();
    }

    public function role($roleName)
    {
        return $roleName ? Str::lower($roleName) == Str::lower($this->role) : $this->role;
    }

    public function org()
    {
        return $this->belongsTo(Organisation::class, 'org_id', 'id');
    }

    public function getFullName()
    {
        return $this->fname . $this->lname;
    }

    public function bank()
    {
        return $this->hasOne(BankAccount::class);
    }
}
