<?php

namespace App\Observers;

use App\Models\Book;
use App\Services\DocumentService as DocService;
use Illuminate\Http\RedirectResponse;
use Illuminate\Support\Facades\Storage;

class BookObserver
{
    /**
     * Handle the Book "created" event.
     *
     * @return void
     */
    public function created(Book $book)
    {
        $this->generateSlug($book);
    }

    /**
     * Handle the Book "updated" event.
     *
     * @return void
     */
    public function updated(Book $book)
    {
        //
    }

    /**
     * Handle the Book "deleted" event.
     *
     * @return void
     */
    public function deleted(Book $book): null|RedirectResponse|bool
    {
        try {
            $this->removeAssocFiles($book);
            // $this->removeRelationships($book);
        } catch (\Throwable $th) {
            return back()->with('error', 'Unable to remove file associations from server : ' . $th->getMessage());
        }
        return true;
    }

    /**
     * Handle the Book "restored" event.
     *
     * @return void
     */
    public function restored(Book $book)
    {
        //
    }

    /**
     * Handle the Book "force deleted" event.
     *
     * @return void
     */
    public function forceDeleted(Book $book)
    {
        //
    }

    //  helpers
    public function removeAssocFiles($book)
    {
        $docService = new DocService();

        if ($book->doc->front_cover && Storage::disk('public')->exists($book->doc->front_cover))
            $docService->deleteImageFromServer($book->doc->front_cover);

        if ($book->doc->back_cover && Storage::disk('public')->exists($book->doc->back_cover))
            $docService->deleteImageFromServer($book->doc->back_cover); // delete back_cover from server

        if ($book->doc->book_file && Storage::disk('public')->exists($book->doc->book_file))
            $docService->deleteImageFromServer($book->doc->book_file); // delete book file from server
    }

    public function removeRelationships($book)
    {
        // $this->book->;
    }

    private function generateSlug(Book $book)
    {
        $bookSlug = str_replace(' ', '-', $book->name);
        $book->slug = strtolower($bookSlug);
        $book->save();
    }
}
