<?php

namespace App\Observers;

use App\Models\Dept;
use App\Organisations\DSPG;

class DeptObserver
{
    /**
     * Handle the Dept "created" event.
     *
     * @param  \App\Models\Dept  $dept
     * @return void
     */
    public function created(Dept $dept)
    {
        if ($dept->org_id == DSPG::id()) {
            $dept->matric_num_fmt = generateMatNumFmtForDSPG($dept->sample_matric_num);
            $dept->pushQuietly();
        }
    }

    /**
     * Handle the Dept "updated" event.
     *
     * @param  \App\Models\Dept  $dept
     * @return void
     */
    public function updated(Dept $dept)
    {
        if ($dept->isDirty('sample_matric_num')) {
            // update this model prop based on its org 
            if ($dept->org_id == DSPG::id()) {
                $dept->matric_num_fmt = generateMatNumFmtForDSPG($dept->sample_matric_num);
                $dept->pushQuietly();
            }
        }
        
        if ($dept->isDirty('slug')) {
            // update this model prop based on its org 
            if ($dept->org_id == DSPG::id()) {
                $dept->update(
                    [
                        'slug' => slugFrom($dept->name) . "__" . auth()->id(),
                    ]
                );
            }
        }


        "__" . auth()->id();
    }

    /**
     * Handle the Dept "deleted" event.
     *
     * @param  \App\Models\Dept  $dept
     * @return void
     */
    public function deleted(Dept $dept)
    {
        //
    }

    /**
     * Handle the Dept "restored" event.
     *
     * @param  \App\Models\Dept  $dept
     * @return void
     */
    public function restored(Dept $dept)
    {
        //
    }

    /**
     * Handle the Dept "force deleted" event.
     *
     * @param  \App\Models\Dept  $dept
     * @return void
     */
    public function forceDeleted(Dept $dept)
    {
        //
    }
}
