<?php

namespace App\Observers;

use App\Models\Book;
use App\Models\Order;
use App\Models\Organisation;
use Illuminate\Support\Facades\DB;

class OrderObserver
{
    /**
     * Handle the Order "created" event.
     *
     * @param  \App\Models\Order  $order
     * @return void
     */
    public function created(Order $order)
    {
        $this->storeSchoolSlugAfterOrderCreated($order);

        $this->storeItemCountAfterOrderCreated($order);

        // updateInventory($order);
    }

    /**
     * Handle the Order "updated" event.
     *
     * @param  \App\Models\Order  $order
     * @return void
     */
    public function updated(Order $order)
    {
        //
    }

    /**
     * Handle the Order "deleted" event.
     *
     * @param  \App\Models\Order  $order
     * @return void
     */
    public function deleted(Order $order)
    {
        //delete the corresponding shoppingcart record in db
        DB::table('shoppingcart')->whereIdentifier($order->identifier)->delete();
    }

    /**
     * Handle the Order "restored" event.
     *
     * @param  \App\Models\Order  $order
     * @return void
     */
    public function restored(Order $order)
    {
        //        
    }

    /**
     * Handle the Order "force deleted" event.
     *
     * @param  \App\Models\Order  $order
     * @return void
     */
    public function forceDeleted(Order $order)
    {
        //
    }

    public function storeSchoolSlugAfterOrderCreated(Order $order)
    {
        $relatedOrg = Organisation::where('name', $order->client?->sch)?->first();
        $order->sch_slug = $relatedOrg?->slug;
        $order->client->org_id = $relatedOrg?->id;
        $order->push();
    }

    public function storeItemCountAfterOrderCreated(Order $order)
    {
        $orderContent = collect(json_decode($order->content));
        $order->items_count = $orderContent->count();
        $order->save();
    }



    /**
     * for all records
     *
     * @return void
     */
    public function storeSchoolSlugAfterOrderComplete__ally()
    {
        $sch_slugs = [];
        foreach (Order::successful()->get() as $order) {
            $relatedOrg = Organisation::find($order->client?->org_id);
            $order->sch_slug = $relatedOrg->slug;
            $order->save();
            $sch_slugs[] = $order->sch_slug;
        }
    }

    /**
     * for all records
     *
     * @return void
     */
    public function storeItemCountAfterOrderCreated__ally()
    {
        foreach (Order::successful()->get() as $order) {
            $order->content = json_decode($order->content);
            $order->content = collect($order->content);
            $order->items_count = $order->content->count();
            $order->save();
        }
    }
}
