<?php

namespace App\Observers;

use App\Models\User;
use App\Enums\UserRoleEnum;
use App\Services\OrgAdminService;
use App\Services\Author\AuthorService;

class UserObserver
{
    /**
     * Handle the User "created" event.
     *
     * @return void
     */
    public function created(User $user)
    {
        // when a user is registered, an [author  or org_admin] profile should be created for them
        ($user->role == UserRoleEnum::ORG) 
            ? (new OrgAdminService)->createOrgAdmin($user)
            : (new AuthorService)->createAuthor($user);
    }

    /**
     * Handle the User "updated" event.
     *
     * @return void
     */
    public function updated(User $user)
    {
        //
    }

    /**
     * Handle the User "deleted" event.
     *
     * @return void
     */
    public function deleted(User $user)
    {
        //
    }

    /**
     * Handle the User "restored" event.
     *
     * @return void
     */
    public function restored(User $user)
    {
        //
    }

    /**
     * Handle the User "force deleted" event.
     *
     * @return void
     */
    public function forceDeleted(User $user)
    {
        //
    }
}
