<?php

namespace App\Services\Author;

use App\Services\File\FileService;
use Illuminate\Http\RedirectResponse;
use App\Http\Requests\UpdateDPRequest;
use Illuminate\Support\Facades\Storage;

class DPService extends FileService
{
    public function updateDP(UpdateDPRequest $request): RedirectResponse
    {
        // dd(reqy()->role, (reqy()->role == 'instructor') and $this->updateAuthorDP());
        try {
            if ((in_array(reqy()->role, ['instructor', 'admin'])) and $this->updateAuthorDP())
                return back()->with('success', 'Picture Uploaded Successfully!!');
        } catch (\Exception $e) {
            $errorMessage = 'Something went wrong while uploading file!' . $e->getMessage();
            return back()->with('error', $errorMessage);
        }


        try {
            // storing unique files in the server
            if ((reqy()->role == 'org_admin') and $this->updateOrgAdminDP())
                return back()->with('success', 'Picture Uploaded Successfully!!');
        } catch (\Exception $e) {
            $errorMessage = 'Something went wrong while uploading admin file!' . $e->getMessage();
            return back()->with('error', $errorMessage);
        }

        return back()->with('error', 'Oops!  Try again');
    }

    // done
    private function storeNewDPToServer($request)
    {
        // main book document
        $new_dpFile = $this->compressFile($request->file('new_dp'));
        $dpFilePath = "files/author_dps/$new_dpFile->name"; // generate file path in server

        return (Storage::disk('public')->put($dpFilePath, $new_dpFile->image)) ? $dpFilePath : null;
    }

    private function storeNewOrgAdminDPToServer($request)
    {
        // main book document
        $new_dpFile = $this->compressFile($request->file('new_dp'));
        $dpFilePath = "files/orgAdmin_dps/$new_dpFile->name"; // generate file path in server

        return (Storage::disk('public')->put($dpFilePath, $new_dpFile->image)) ? $dpFilePath : null;
    }


    private function deleteOldPhotoFromServer()
    {
        $photoFilePath = author()->dp;
        try {
            Storage::disk('public')->exists($photoFilePath) ? Storage::disk('public')->delete($photoFilePath) : null;
        } catch (\Exception $e) {
            return back()->with('error', 'Photo could not be Overwritten');
        }
    }


    private function updateAuthorDP()
    {
        // storing unique files in the server
        if (reqy()->author->dp) {
            $this->deleteOldPhotoFromServer();
        } // now delete all existing files

        reqy()->author->dp = $this->storeNewDPToServer(request()); //store to server and update db

        return reqy()->author->save();
    }

    private function updateOrgAdminDP()
    {
        // storing unique files in the server
        if (reqy()->orgAdmin->dp) {
            $this->deleteOldPhotoFromServer();
        } // now delete all existing files

        reqy()->orgAdmin->dp = $this->storeNewOrgAdminDPToServer(request()); //store to server and update db

        return reqy()->orgAdmin->save();
    }

}
