<?php

namespace App\Services;

use App\Http\Requests\DocumentUploadRequest;
use App\Models\Book;
use App\Models\Product;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;

class DocumentService
{
    public $request;

    /**
     * This method processes multiple photos at once for any given product, for the purpose of storing or uploading into the db.
     *
     * It could be a new producct or an already existing product
     */
    public function __construct()
    {
        // if (!request() instanceof DocumentUploadRequest)
        //    $this->validateRequest();
        // $this->request = request();
    }

    /**
     * The only validation we are doing here is for images. That is the only thing that concerns us (as a PhotoService)
     */
    public function validateRequest()
    {
        // request()->validate([
        //    'main_document.*' => 'file|mimes:pdf,doc,docx|max:8192',
        //    'front_cover.*' => 'image|mimes:jpeg,png,jpg,gif,svg|max:8192'

        // ]);
    }

    /**
     * please note that when consuming this api, the request must have an 'images' key that houses all the images
     * eg. $request->images should hold all images. follow this convention
     */
    public function processMultiplePhotos(DocumentUploadRequest|Request $request, Book $product): bool
    {
        // Loop through the uploaded images and save them to your desired location
        foreach ($request->file('images') as $image) {
            // store image to desired location in file system publicly or privately via $image->storeAs('private/products/images', $fileName);
            $filepath = $this->uploadToServer($image);
            $this->storePhotoInDB($product, $filepath);
        }

        return true; // if code gets here then return true, else it would have returned false already
    }

    /**
     * @param $product . The associated product for this photo
     */
    public function storePhotoInDB($product, $filepath): bool
    {
        // assiging to first product and any random User in db for testing purposes
        $photoUpload = $product->photos()->create([
            // when storing the url in the db, it is neccessary to remove the disk path // which resides in the root of your storage ie. public
            'url' => str_replace('public/', '', $filepath),
            'caption' => basename($filepath), //suposed filename
            'info' => fake()->name(),
            'user_id' => auth()->user()->id,
        ]);

        return $photoUpload ? true : false;
    }

    /**
     * @return string $filepath
     */
    public function uploadToServer($image, $path = 'products/images'): string|bool
    {
        $filepath = $image->store("public/$path");

        return $filepath ?: false;
    }

    public function deleteImageFromServer(string $photoUrl): bool|string
    {
        return (Storage::disk('public')->exists($photoUrl)) ? Storage::disk('public')->delete($photoUrl) : 'nothing happened';
        // Storage::delete(['upload/test.png', 'upload/test2.png']); // Delete Multiple files this way
    }
}
