<?php

namespace App\Services;

use App\Models\Order;
use Illuminate\Database\Eloquent\Collection;

class OrderService
{

    public function __construct()
    {
    }

    /**
     * query db for orders belonging to a specific organisation
     * returns a filtered result from the db query level.
     * efficient in storage
     * @param string|null $duration
     * @param string|null $sch_slug
     * @return Collection
     */
    public function orderTotal(string|null $duration, string|null $sch_slug = null): Collection
    {
        if ($duration) {
            try {
                match ($duration) {

                    'today' => $duration = [now()->startOfDay(), now()->endOfDay()],
                    'yesterday' => $duration = [now()->subDay()->startOfDay(), now()->subDay()->endOfDay()],
                    'this_week' => $duration = [now()->startOfWeek(), now()->endOfWeek()],
                    'last_week' => $duration = [now()->subWeek()->startOfWeek(), now()->subWeek()->endOfWeek()],
                    'this_month' => $duration = [now()->startOfMonth(), now()->endOfMonth()],
                    'last_month' => $duration = [now()->subMonth()->startOfMonth(), now()->subMonth()->endOfMonth()],
                    'this_year' => $duration = [now()->startOfYear(), now()->endOfYear()],
                    'last_year' => $duration = [now()->subYear()->startOfYear(), now()->subYear()->endOfYear()],
                    default => throw new \Exception("Error Processing Request : '$duration' is not specified in your match code block", 1)
                };
            } catch (\Throwable $th) {
                dd($th->getMessage());
            }
        }


        if ($sch_slug) {
            $sch_slug = strtoupper($sch_slug);

            $requestedOrders = $duration
                ? Order::successful()->whereBetween('created_at', $duration)->where('sch_slug', $sch_slug)->get()
                : Order::successful()->where('sch_slug', $sch_slug)->get();
        } else {
            $requestedOrders = $duration
                ? Order::successful()->whereBetween('created_at', $duration)->get()
                : Order::successful()->get();
        }

        return $requestedOrders;
    }

    /**
     * get the total number of books sold in a school or all schools
     *
     * @param string $duration
     * @param string $sch_slug
     * @return int
     */
    public function orderTotalCount($duration,  $sch_slug = null): int
    {
        return   $this->orderTotal($duration, $sch_slug)->count();
    }

    /**
     * filters a list of orders to a specific institution/organisation eg. dspg
     * 
     * use this on a collection after retrieving from db
     *
     * avoid using this method, it has to get all records before it returns the required result
     * @param array $foundOrders expects a unique array of orders (non duplicates) made in a specific duration
     * @param string $orgSlug expects a valid $orgSlug stored in the db eg. 'DSPG', 'LASUED', 'DSPGA'
     * @return void
     */
    public function filterOrdersForOrg(Collection $requestedOrders, $orgSlug = 'DSPG'): Collection
    {
        /**
         * refers to the filtered array that contains only orders for selected school
         * that were made in a specific duration
         * @var mixed $filtered
         */
        return $filtered = $requestedOrders->where('sch_slug', $orgSlug);
    }
}
