<?php

namespace App\Services;

use App\Models\Organisation as Org;
use Illuminate\Support\Str;
use App\Enums\UserRoleEnum as Role;

/**
 * This is a godclass for all the form requests fields that will
 * be used in the app.
 * Make sure all field names are the same as what they are on the forms you created, don't use
 * direct db column names, for your attributes, when you used a different field name in your code
 */
class RegistrationTestService
{

    public function __construct()
    {

    }


    /**
     * returns the required form fields for creating a user (for instructor)
     * for existing organisations
     * @return array
     */
    public function createInstructorForExistingOrgs(): array
    {
        return [
            'name' => substr(fake()->name(), 0, 10),
            'email' => fake()->unique()->safeEmail(),
            'email_verified_at' => now()->addMinutes(2),
            'password' => '$2y$10$92IXUNpkjO0rOQ5byMi.Ye4oKoEa3Ro9llC/.og/at2.uheWG/igi',
            'password_confirmation' => '$2y$10$92IXUNpkjO0rOQ5byMi.Ye4oKoEa3Ro9llC/.og/at2.uheWG/igi',
            // password
            'remember_token' => Str::random(10),
            //testing only the instructor case scenario
            'is_admin' => false,
            'role' => Role::DEFAULT ,
            //organisation field => db:col('org_id') | Default value is 1 => dspg
            'organisation' => Org::first()->id,
            'created_at' => now(),
            'terms' => true,
        ];
    }

    /**
     * 
     * returns the required form fields for creating a user (for instructor)
     * for new organisations
     * @return array
     */
    public function createInstructorForNewOrgs(): array
    {
        return [
            // constants
            'name' => fake()->name(),
            'email' => fake()->unique()->safeEmail(),
            'email_verified_at' => now()->addMinutes(2),
            'password' => '$2y$10$92IXUNpkjO0rOQ5byMi.Ye4oKoEa3Ro9llC/.og/at2.uheWG/igi',
            'password_confirmation' => '$2y$10$92IXUNpkjO0rOQ5byMi.Ye4oKoEa3Ro9llC/.og/at2.uheWG/igi',
            // 'remember_token' => Str::random(10),
            'is_admin' => false,
            'created_at' => now(),
            'terms' => true,

            //testing only the instructor case scenario
            'role' => Role::DEFAULT ,
            //org id would not be provided on form cos it's a new org
            'organisation' => 'new',
            'organisation_name' => generateRandomOrgName(),
        ];
    }

    /**
     * 
     * returns the required form fields for creating a user with role: 'admin'
     *Global admin for the app (dev)
     * @return array
     */
    public function createAppAdmin()
    {

        return [
            'name' => fake()->name(),
            'email' => fake()->unique()->safeEmail(),
            'email_verified_at' => now(),
            'password' => '$2y$10$92IXUNpkjO0rOQ5byMi.Ye4oKoEa3Ro9llC/.og/at2.uheWG/igi',
            'password_confirmation' => '$2y$10$92IXUNpkjO0rOQ5byMi.Ye4oKoEa3Ro9llC/.og/at2.uheWG/igi',
            // password
            'remember_token' => Str::random(10),
            //testing only the instructor case scenario
            'is_admin' => true,
            'role' => Role::ADMIN,
            //organisation field => db:col('org_id') | Default value is 1 => dspg
            'organisation' => Org::first()->id,
            'terms' => true,

            'created_at' => now(),

        ];
    }


    /**
     * 
     * returns the required form fields for creating a user 
     * with role : org_admins for existing  organisatins
     *
     * @return array
     */
    public function createOrgAdminForExistingOrgs()
    {

        return [
            'name' => fake()->name(),
            'email' => fake()->unique()->safeEmail(),
            'email_verified_at' => now(),
            'password' => '$2y$10$92IXUNpkjO0rOQ5byMi.Ye4oKoEa3Ro9llC/.og/at2.uheWG/igi',
            'password_confirmation' => '$2y$10$92IXUNpkjO0rOQ5byMi.Ye4oKoEa3Ro9llC/.og/at2.uheWG/igi',
            // password
            'remember_token' => Str::random(10),
            //testing only the instructor case scenario
            'is_admin' => false,
            'role' => Role::ORG,
            //organisation field => db:col('org_id') | Default value is 1 => dspg
            'created_at' => now(),
            'terms' => true,
            'organisation' => Org::first()->id,
            // 'organisation' => 1,
            // compulsory fields for this scenario
            'sch_ims_code' => Org::first()->ims_code,

        ];
    }

    /**
     * 
     * returns the required form fields for creating a user 
     * with role : org_admins for existing  organisatins
     *
     * @return array
     */
    public function createOrgAdminForNewOrgs()
    {
        return [
            'name' => fake()->name(),
            'email' => fake()->unique()->safeEmail(),
            'email_verified_at' => now(),
            'password' => '$2y$10$92IXUNpkjO0rOQ5byMi.Ye4oKoEa3Ro9llC/.og/at2.uheWG/igi',
            'password_confirmation' => '$2y$10$92IXUNpkjO0rOQ5byMi.Ye4oKoEa3Ro9llC/.og/at2.uheWG/igi',
            // password
            'remember_token' => Str::random(10),
            //testing only the instructor case scenario
            'is_admin' => false,
            //organisation field => db:col('org_id') | Default value is 1 => dspg
            'created_at' => now(),
            'terms' => true,

            //testing only the instructor case scenario
            'role' => Role::ORG,
            //org id would not be provided on form cos it's a new org
            'organisation' => 'new',
            'organisation_name' => generateRandomOrgName(),
            // ommitted
            // 'sch_ims_code' => $firstOrg->ims_code,
        ];
    }
}
