<?php

namespace App\Services;

use App\Models\PG;
use App\Models\Order;
use App\Models\Developer;
use App\PaymentGatewayServices\Flutterwave;

class RemittanceService
{

    public function __construct()
    {
    }

    public function generateAuthorRemitForOrderItem(array &$orderRemit, $orderItem)
    {
        $orderRemit['authors'] += $orderItem->options->original_price;
    }

    public function generateTotalOrderAmount(array &$orderRemit, $orderItem)
    {
        $orderRemit['total'] += $orderItem->price + $orderItem->tax; //inflated price + tax (if any)
    }

    /**
     * computes the payment gateway service remit/fee
     * 
     */
    public function computePGSRemitForOrder($pendingOrder)
    {

        // step 1 choose pgs (already done when creating pendingOrder)
        // step2: compute value based on selected pgs fee
        $pgRate = $pendingOrder->pgs->rate;

        return $pendingOrder->total_amount * $pgRate;
    }


    /**
     * this returns the total author's remit for any given order
     * 
     * it works by passing through all the cartItems in the order and summing up all the author remits 
     * this must always return an int. it should never return a float.
     * 
     * @return integer
     */
    public function computeAuthorRemitForOrder(Order $newPendingOrder): int
    {
        $orderRemit = ['authors' => 0.00];

        foreach ($newPendingOrder->getBasket() as $orderItem)
            $this->generateAuthorRemitForOrderItem($orderRemit, $orderItem);

        return $orderRemit['authors'];
    }


    /**
     * this returns the total amount paid for any given order
     * 
     * it works by passing through all the cartItems in the order and summing up all the prices 
     * this must always return an int. it should never return a float.
     * 
     * @return integer
     */
    public function computeTotalAmountForOrder(Order $newPendingOrder): int
    {
        return $newPendingOrder->total_amount;
    }

    /**
     * this returns the total amount paid for any given order
     * 
     * it works by passing through all the cartItems in the order and summing up all the prices 
     * this must always return an int. it should never return a float.
     * 
     * @return integer
     */
    public function computeTotalAmountForOrderV2(Order $newPendingOrder): int
    {
        $orderRemit = ['total' => 0.00];

        foreach ($newPendingOrder->getBasket() as $orderItem)
            $this->generateTotalOrderAmount($orderRemit, $orderItem);

        return $orderRemit['total'];
    }

    public function computeSchRemitForOrder(Order $newPendingOrder): int
    {
        $countItemsInThisOrder = $newPendingOrder->getBasket()->count();
        return $schRemitForOrder = $countItemsInThisOrder * getSchoolVAT();
    }


    public function computeDevRemitForOrder(Order $order): float
    {
        $authorRemit = $this->computeAuthorRemitForOrder($order);
        $schoolRemit = $this->computeSchRemitForOrder($order);
        $pgsRemit = $this->computePGSRemitForOrder($order);
        $allOtherRemits = $authorRemit + $schoolRemit + $pgsRemit;

        $devRemit = $order->total_amount - $allOtherRemits;

        $order->update([
            'authors_remit' => $authorRemit,
            'school_remit' => $schoolRemit,
            'pgs_remit' => $pgsRemit,
            'dev_remit' => $devRemit,
        ]);

        return $devRemit;
    }


    public function shareFunds(Order $order): bool
    {
        $order->dev_remit = $this->computeDevRemitForOrder($order);
        // return $order->push(); //old and working method
        return $order->pushQuietly(); //new and untested method

    }

    public function validateRemits($order)
    {
        $sumOfAllRemits = $order->pgs_remit + $order->school_remit + $order->authors_remit + $order->dev_remit;

        return $validatedTotal = (round($order->total_amount, 6) == round($sumOfAllRemits, 6));
    }

}
