<?php
namespace App\Traits\AppTraits;

use App\Models\PG;
use App\Models\Book;
use App\Models\Client;
use App\Mail\OrderFundShareFailed;
use Illuminate\Support\Facades\DB;
use App\Mail\OrderConfirmationMail;
use App\Services\RemittanceService;
use Illuminate\Support\Facades\Mail;
use Gloudemans\Shoppingcart\Facades\Cart;

trait PaymentGatewayTrait
{
    public function generateViewForSuccessfulPayment($relatedOrder)
    {

        Cart::destroy(); //important: empty cart after flw has confirmed order to be successful in db

        DB::beginTransaction();

        try {
            $selectedPGS = PG::where('id', $relatedOrder->pgs_id)->first()->name;

            $pgs_tx_id = '';

            match ($selectedPGS) {
                'flutterwave' => $pgs_tx_id = session('paymentParams.flwTrxId'),

                'rexpay' => $pgs_tx_id = 'rexpay_' . $relatedOrder->id,

                default => $pgs_tx_id = 'pgs_' . $relatedOrder->id,
            };

            $relatedOrder->pgs_tx_id = $pgs_tx_id; //cause it's a unique col

            $relatedOrder->status = 'successful'; //update order status

            updateInventory($relatedOrder);

            $relatedOrder->save();

            // this block of code is the same for all payment gateways, make it DRY
            $relatedClient = Client::find($relatedOrder->client->id);
            Mail::to($relatedClient->email)->send(new OrderConfirmationMail($relatedOrder));

            $relatedContent = $relatedOrder->getBasket()->toArray();
            $booksPurchased = array_map(fn($item) => Book::find($item->id), $relatedContent);

            //important db transaction
            $shareFunds = (new RemittanceService)->shareFunds($relatedOrder);
            if (!$shareFunds)
                Mail::to(env('DEV_EMAIL1'))->send(new OrderFundShareFailed($relatedOrder)); //mail developer

            DB::commit();

            session()->remove('paymentParams'); //new and untested method

            return view('snit.thankyou.index', [
                'order_id' => $relatedOrder->identifier,
                'ordered_content_models' => $booksPurchased
            ]);

        } catch (\Throwable $th) {
            DB::rollback();
            echo $th->getMessage();
            //throw $th;
        }


    }

}
