<?php
namespace App\Traits\ModelTraits\Book;

use Illuminate\Database\Eloquent\Casts\Attribute;

/**
 * used to get special (formatted)attributes from the db
 */
trait BookAccessors
{
    // protected function isbn(): Attribute
    // {
    //     return Attribute::make(
    //         get: fn(string $value) => implode('-', str_split($value, 4))
    //     );
    // }

    # =====================================================================
    # ========================DYNAMIC ATTRIBUTES===========================
    # =====================================================================

    /**
     * get slug in Snake Case as attribute
     *
     * @return string
     */
    public function getSlugInSnakeCaseAttribute(): string
    {
        return str_replace('-', '_', $this->slug);
    }

    /**
     * get slug as variable name
     *
     * @return string
     */
    public function getSlugAsVariableAttribute(): string
    {
        return $this->getSlugInSnakeCaseAttribute();
    }

    /**
     * get ebook profit as attribute
     * @return void
     */
    public function getEbookProfitAttribute()
    {
        return $this->service->presentTotalEbookSalesProfit() ?? 'zilch';
    }

    public function getHardcoverProfitAttribute()
    {
        return $this->service->presentTotalHardcoverSalesProfit() ?? 'zilch';
    }

    public function getTotalProfitAttribute()
    {
        return $this->ebookProfit + $this->hardcoverProfit;
    }

    public function getTotalSoldAttribute()
    {
        return $this->service->totalHardcoverOrders()->count() + $this->service->totalEbookOrders()->count();
    }

    public function getTotalSalesValueAttribute()
    {
        return $this->service->presentTotalHardcoverSales() + $this->service->presentTotalEbookSales();
    }
}