<?php

namespace App\Traits\ModelTraits\Book;

use App\Models\Developer;
use App\Services\Book\BookService;


trait BookHelpers
{

    public function getTotalSalesGenerated()
    {
        $totalFound = $this->service->getCollectionOfTotalBookSold();
        $sumOfPrices = $totalFound->sum('price');

        return $sumOfPrices;
    }

    /**
     * For Authors, this shows them how much each book has generated in net sales excluding service charges and indirect taxes paid by students
     */
    public function presentTotalSalesGenerated($duration = null)
    {
        switch ($duration) {

            case 'today':
                $sumOfPrices = $this->service->getCollectionOfTotalBookSoldToday()->sum('options.original_price');
                break;

            case 'this_week':
                $sumOfPrices = $this->service->getCollectionOfTotalBookSoldThisWeek()->sum('options.original_price');
                break;

            case 'this_month':
                $sumOfPrices = $this->service->getCollectionOfTotalBookSoldThisMonth()->sum('options.original_price');
                break;

            case 'this_year':
                $sumOfPrices = $this->service->getCollectionOfTotalBookSoldThisYear()->sum('options.original_price');
                break;

            // past data

            case 'yesterday':
                $sumOfPrices = $this->service->getCollectionOfTotalBookSoldYesterday()->sum('options.original_price');
                break;

            case 'last_week':
                $sumOfPrices = $this->service->getCollectionOfTotalBookSoldLastWeek()->sum('options.original_price');
                break;

            case 'last_month':
                $sumOfPrices = $this->service->getCollectionOfTotalBookSoldLastMonth()->sum('options.original_price');
                break;

            case 'last_year':
                $sumOfPrices = $this->service->getCollectionOfTotalBookSoldLastYear()->sum('options.original_price');
                break;

            default:
                $sumOfPrices = $this->service->getCollectionOfTotalBookSold()->sum('options.original_price');
                break;
        }
        return $sumOfPrices;
    }

    /**
     * For Authors, this shows them how much each book has generated for the past month (ie. past 30 days) in gross profit less service charges and indirect taxes paid by students
     */
    public function presentTotalSalesGeneratedThisMonth()
    {
        return $sumOfPrices = $this->service->getCollectionOfTotalBookSold()->sum('options.original_price');
    }

    /**
     * for developer
     */
    public function getTotalEbookSales()
    {
        return $this->totalEbookSales()->sum('price');
    }


    // for authors
    public function ebookPrice(): float
    {
        return $this->price * (.55); // i.e the student pays 60%(or 40% discount) of the actual book price 
    }

    // For buyers: present price for ebook with included 5.5% trx charge and additional school and flw price
    public function presentEbookPrice(): int
    {
        return $newEbookPrice = (int) $this->updatePriceWithServiceCharge('ebook');
    }

    // fees included price for hardcover
    public function presentPrice(): int
    {
        return $newPrice = (int) $this->updatePriceWithServiceCharge();
    }

    /**
     *Adds SchoolVAT to either eBook or Hardcover listings in the bookstore
     * 
     *  @param string $bookType refers to either hardcover or ebook
     * 
     * the default bookType == 'hardcover' , hence: always  explicitly specify bookType for 'ebooks'
     * 
     * NB: this uses DSPG rate by default
     */
    public function addSchoolVAT($bookType = 'hardcover'): float
    {
        return $bookType == 'hardcover'
            ? ($priceWithSchoolVAT = $this->price + getSchoolVAT() )
            : ($priceWithSchoolVAT = $this->ebookPrice() + getSchoolVAT() );
    }

    /**
     * the default bookType == 'hardcover'
     */
    public function updatePriceWithServiceCharge($bookType = 'hardcover'): int
    {
        $priceWithSchoolVAT = ($bookType == 'ebook') ? $this->addSchoolVAT('ebook') : $this->addSchoolVAT();

        $trxFee = $priceWithSchoolVAT * Developer::TOTAL_SERVICE_FEE_IN_PERC; // 5.5% trx fee (50/50 with flw)

        $newPrice = $priceWithSchoolVAT + $trxFee; //add 5.5% trx fee to ebookPrice

        return roundUpToNearestTens($newPrice);
    }



    public function service()
    {
        return $this->service = new BookService($this);
    }
}
