<?php
namespace App\Traits\RequestTraits\UserSignup;

use App\Models\Country;
use App\Enums\UserRoleEnum as Role;
use App\Models\Organisation as Org;

trait SignupHelpers
{
    // helpers
    protected function handleOrganisationLogicForOrgAdminWithExistingOrg()
    {
        $newOrExistingOrgID = $this->validData['organisation'] ?? 0;
        $matchedOrgID = $this->getOrgIDFromIMSCode(); //only for org_admins
        // comparison should only be done when we are sure that $matchedOrdID != 0
        if (($matchedOrgID > 0) && ((int) $matchedOrgID === (int) $newOrExistingOrgID)) {
            $this->validData['org_id'] = $newOrExistingOrgID; // update this->validData array
        } else {
            // create an error for the controller to handle
            $this['errors'] = [
                'ims code mismatch'
            ];
            return back(); // to the controller
        }
    }


    /**
     * creates a new organisation and assign id to  $this->validData['organisation']
     * @return integer returns the id of the new organisation
     */
    protected function createNewOrgAndGetID(): int
    {
        // if a new name was provided, create new org in db | [NB: they're mutually exclusive]
        if ($this->organisation_name) {
            // dd('new stuff');
            $newOrg = Org::create([
                //unique fields
                'name' => $this->organisation_name,
                'ims_code' => $this->sch_ims_code,

                //generate slug from ims code
                'slug' => orgSlugFrom($this->sch_ims_code),
                //default to Nigeria
                'country_id' => Country::first()->id,
            ]);

            if ($newOrg && $newOrg->selfValidateIMSCode())
                return $this->validData['organisation'] = $newOrg->id;
        }
        return 0;
    }


    /**
     * for existing organisations, get the org->id for the IMS code provided by new user
     *
     * @return integer
     */
    protected function getOrgIDFromIMSCode(): int
    {
        /**
         * Supposing the org->ims_code provided is a valid one, let's get the related model id
         * for admins, this is the matched org id that must be thesame with 
         * whatever organisation that they picked from the  >select> drop down
         */
        // dd(Org::orgCodes());
        if ($this->organisation && ($this->organisation !== 'new')) //one_liner
            foreach (Org::IMSCodes() as $id => $orgCode) //one_liner
                if ($this->sch_ims_code == $orgCode) //one_liner
                    return $matchedOrgID = $id; //find d first match and return it asap
        return 0; // normally should be 0, cos it's not found
    }

}