<?php

namespace App\Traits\RequestTraits\UserSignup;

use App\Models\User;
use Illuminate\Support\Facades\Hash;
use Illuminate\Validation\Validator;
use App\Enums\UserRoleEnum as Role;


trait SignupPostValidation
{

    /**
     * important lesson, When using the withValidator(), don't use return keyword else it'll 
     * hand over control to the controller for this request
     * in a nutshell all methods invoked should have a return type of void;
     * @param Validator $validator
     * @return void
     */
    public function withValidator(Validator $validator)
    {
        // do more post validation checks as needs be.
        $validator->after(function (Validator $validator) {
            $this->denyUserFromRegisteringAsAppAdmin();
            $this->validData = array_merge($validator->safe()->all(), $this->validData);

            // for all new users[instructors, org_admins] with new organisations
            if ($this->organisation == 'new')
                $this->createNewOrgAndGetID();

            // for all new users[instructors, org_admins] with existing organisations
            if ($this->role == Role::ORG and $this->organisation != 'new')
                $this->handleOrganisationLogicForOrgAdminWithExistingOrg();
        });
    }


    /**
     * handles user creation both for authors and org_admin purposes
     *
     * @return User|null
     */
    public function createUser(): User|null
    {
        // for both org_admins and authors
        return User::create([
            'name' => $this->validData['name'],
            'email' => $this->validData['email'],
            'password' => Hash::make($this->validData['password']),
            // you only become an org_admin when you provide a valid ims code
            'role' => $this->validData['role'],
            // you are linked to an org. using your org_admin or selected org from form
            'org_id' => $this->validData['organisation'] ?? $this['organisation']
        ]);
    }

    public function denyUserFromRegisteringAsAppAdmin()
    {
        if ($this->role == Role::ADMIN)
            $this['errors'] = [
                'Unauthorized attempted to register as admin'
            ];
        return back();
    }
}
