<?php
namespace App\Traits\RequestTraits\UserSignup;

use App\Models\User;
use Illuminate\Validation\Rules;
// use Illuminate\Validation\Rules\Password;
use App\Enums\UserRoleEnum as Role;


trait SignupRules
{
    public function getDefaultRules()
    {
        return $this->defaultRules = [
            // personal info
            'name' => ['required', 'string', 'min:5', 'max:255'],
            'email' => ['required', 'string', 'email', 'max:255', 'unique:' . User::class],
            'role' => ['required'],
            'password' => ['required', 'confirmed', Rules\Password::defaults()],
            'terms' => ['required'],
        ];
    }


    private function getRulesForNewInstructorsUnderExistingOrgs(): array
    {
        return $rulesForNewInstructorsUnderExistingOrgs = array_merge($this->defaultRules, [
            'organisation' => ['required'],
        ]);
    }

    private function getRulesForNewInstructorsUnderNewOrgs(): array
    {
        return $rulesForNewInstructorsUnderNewOrgs = array_merge($this->defaultRules, [
            'organisation_name' => ['required'],
        ]);
    }


    private function getRulesForNewOrgAdminsUnderExistingOrgs(): array
    {
        return $rulesForNewOrgAdminsUnderExistingOrgs = array_merge($this->defaultRules, [
            'organisation' => ['required'],
            'sch_ims_code' => ['required_if:role,org_admin', 'exists:organisations,ims_code'],
            // omitted : irrelevant
            // 'organisation_name' => ['required_if:organisation,new'],
        ]);
    }

    private function getRulesForNewOrgAdminsUnderNewOrgs(): array
    {
        return $rulesForNewOrgAdminsUnderNewOrgs = array_merge($this->defaultRules, [
            'organisation_name' => ['required_if:organisation,new'],
        ]);
    }

    /**
     * generates validation rules based on different user registration scenarios
     *
     * @return void
     */
    private function ruleGen(): array
    {
        $getMyRules = array();
        /*
        |--------------------------------------------------------------------------
        | INSTRUCTOR RULES
        |--------------------------------------------------------------------------
        */
        // for new instructors with existing organisations
        if ($this->role === Role::VENDOR and $this->organisation !== 'new') {
            return $getMyRules = $this->getRulesForNewInstructorsUnderExistingOrgs();
            // dd('army', $getMyRules);
        }
        // for new instructors with new organisations
        if ($this->role === Role::VENDOR and $this->organisation === 'new') {
            return $getMyRules = $this->getRulesForNewInstructorsUnderNewOrgs();
        }
        /*
        |--------------------------------------------------------------------------
        | ORG_ADMIN RULES
        |--------------------------------------------------------------------------
        */
        // for new org_admins with existing organisations
        if ($this->role === Role::ORG and $this->organisation !== 'new')
            return $getMyRules = $this->getRulesForNewOrgAdminsUnderExistingOrgs();

        // for new org_admins with new organisations
        if ($this->role === Role::ORG and $this->organisation == 'new')
            return $getMyRules = $this->getRulesForNewOrgAdminsUnderNewOrgs();

        $whichUser = PHP_EOL . 'what kind of user are you?' . PHP_EOL;
        return [$whichUser]; //someone trying to register as admin
    }
}
