<?php

use App\Models\Country;
use App\Models\Currency;
use App\Models\Language;
use App\Models\Organisation;
use App\Models\User;
use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

return new class extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::create('authors', function (Blueprint $table) {
            $table->id();
            $table->string('fname', 20)->nullable();
            $table->string('lname', 20)->nullable();
            $table->string('info')->nullable(); // masters or PHD info goes here along with their institution data

            // nullable data
            $table->string('phone', 15)->nullable();
            $table->string('address', 100)->nullable();
            $table->string('state', 50)->nullable();
            $table->string('zipcode', 10)->nullable();
            $table->char('gender', 1)->nullable(); // M | F
            $table->string('dp')->nullable();

            // foreign key checks
            $table->foreignIdFor(User::class);
            $table->foreignIdFor(Organisation::class, 'org_id')->nullable();
            $table->foreignIdFor(Country::class)->nullable();
            $table->foreignIdFor(Currency::class)->nullable();
            $table->foreignIdFor(Language::class)->nullable();

            // new mig additions
            $table->timestamp('last_paid')->nullable();
            $table->boolean('is_paid')->default(false);
            // eol new mig additions
            $table->timestamp('created_at')->default(now());
            $table->timestamp('updated_at')->nullable();
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('authors');
    }
};
