<?php

use App\Models\User;
use App\Models\Organisation;
use Illuminate\Support\Facades\Schema;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Database\Migrations\Migration;

return new class extends Migration {
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::create('depts', function (Blueprint $table) {
            $table->id();

            // foreign keys
            $table->foreignIdFor(User::class);
            $table->foreignIdFor(Organisation::class, 'org_id');
            $table->string('institute', 100); // eg name of Uni, Poly, College of Edu etc.
            $table->string('name', 100); // eg computer science
            $table->string('slug')->unique()->nullable();
            $table->enum('program_type', ['pt', 'ft'])->default('ft'); // eg. part time, full time
            $table->string('dept_level', 20)->nullable(); // eg. NDI, 300L, IT, done etc
            $table->string('sample_matric_num', 30); // eg. com/nd/22/23/0043 
            $table->string('matric_num_fmt')->nullable();

            // there will be a btn in the instructor dashboard that says, configure mat nos. for new session
            // rules for auto config of mat nos.
            // 1. only nd1 and hnd1 are migrateable (only change the level to increment by 1, eg. nd1 becomes nd2)
            // 2. nd2 and hnd2 are handled differently
            // i.e nd2 are put on 'IT' and hnd2 are put on 'DONE'

            $table->timestamp('created_at')->default(now());
            $table->timestamp('updated_at')->nullable();
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('depts');
    }
};
