<?php

namespace Database\Seeders;

use App\Enums\UserRoleEnum as Role;
use App\Models\User;
use Illuminate\Auth\Events\Registered;
use Illuminate\Database\Seeder;
use Illuminate\Support\Facades\Hash;

class UserSeeder extends Seeder
{
    /**
     * Run the database seeds.
     *
     * @return void
     */
    public function run()
    {
        $this->createAdmins();
        $this->createInstructors();
        $this->createOrgs();

    }


    public function createAdmins()
    {
        // admin users
        $admins = [
            // 'wftester465',
            'jbstiles100',
            'danmoney6',
        ];
        foreach ($admins as $admin) {
            $userCreated = User::create([
                'name' => $admin,
                'email' => "$admin@gmail.com",
                'password' => Hash::make($admin),
                'is_admin' => true,
                'role' => Role::ADMIN,
                'org_id' => 1,
                // simulating dspg
            ]);
        }

    }

    public function createInstructors()
    {
        // normal instructor users
        $instructors = [
            'konwei',
            'chijoke',
            'yoro',
            'nwanze',
            'steve',
            'obukata',
            'jonathan',
        ];
        foreach ($instructors as $instructor) {
            $userCreated = User::create([
                'name' => $instructor,
                'email' => "$instructor@gmail.com",
                'password' => Hash::make($instructor),
                'is_admin' => false,
                'role' => Role::DEFAULT ,
                // simulating dspg for org_id
                'org_id' => 1,
            ]);
        }
    }

    public function createOrgs()
    {
        // organisation users
        $orgs = [
            'emefiene',
            'nkem',
            'mary',
        ];
        foreach ($orgs as $org) {
            $userCreated = User::create([
                'name' => $org,
                'email' => "$org@gmail.com",
                'password' => Hash::make($org),
                'is_admin' => false,
                'role' => Role::ORG,
                'org_id' => 1,
                // simulating dspg
            ]);
        }
    }
}
