@extends('snit.layouts.app')

@section('content')
    <!-- Bordered Table -->
    @if (count($allBooks) < 1)
        <p class="my-5 py-5 text-sm text-center text-white fw-light"> 
            Oops! You have no books <br><br>
            <a class="underline text-sm text-blue-400" href="{{ route('books.create') }}">
            &rArr;    Add Your First Book
            </a>
        </p>
    @else
        <div class="card mx-4 mt-4">
            <div class="d-flex justify-content-between">
                <h3 class="card-header mx-2 h5">My Books</h3>
                <div class="d-block m-4">
                    <button class="btn btn-sm btn-primary"><a href="{{ route('bookstore.index') }}" class='text-white'>View
                            Store</a></button>
                </div>
            </div>
            <div class="mx-2 my-0">
                <div class="d-block mx-4">
                    <a href="{{ route('books.create') }}" class='text-white'>
                        <button class="btn btn-success btn-sm">
                            <i class="menu-icon tf-icons bx bx-plus-circle"></i>
                            Add New
                        </button>
                    </a>
                    <div class="mt-1 mb-0 text-lg w-fit px-3 py-1 text-white bg-blue-400">
                       <span class="text-xs"> All Books: {{ count($allBooks) }}</span>
                    </div>
                </div>

            </div>

            <div class="card-body">
                <div class="table-responsive text-nowrap">
                    <table class="table py-5">
                        <col>
                        <col>
                        <col>
                        <col>
                        <col>
                        <col>
                        <col>
                        <col>
                        <col>

                        {{-- profit margins --}}
                        <thead>
                            <tr class="bg-primary">
                                @foreach ($table_headers as $heading)
                                    <th class='max-w-fit text-xs px-1 '>
                                        <small class="text-white">{!! $heading !!}</small>
                                    </th>
                                @endforeach
                            </tr>
                        </thead>
                        <tbody>
                            @foreach ($allBooks as $book)
                                <tr
                                    class="@empty($book->doc) bg-red-500  text-white @else
                                        hover:bg-green-100 hover:text-black cursor-pointer
                                     @endempty
                                    ">

                                    {{-- s/n --}}
                                    <td class="px-0 text-center">{{ $loop->iteration }}</td>
                                    {{-- file --}}
                                    <td class='ps-2'>
                                        {!! (Storage::disk('public')->exists($book->doc?->book_file) or $book->doc?->book_file)
                                            ? ' <i class="bx bx-check-circle text-xs text-green-500"></i>'
                                            : '<i class="bx bx-x text-xs text-red-500"></i>' !!}
                                    </td>

                                    {{-- front cover image --}}
                                    <td class='px-0'>
                                        <img src="/storage/{{ $book->doc->front_cover ?? ' ' }}" width=40
                                            alt="{{ $book->doc->front_cover ?? 'No Image' }}" class="src" />
                                    </td>


                                    
                                    <td class='px-0'>
                                        <i class="fab fa-angular fa-lg text-danger me-3"></i>
                                        <strong>{{ $book->inventory ?? 0 }}</strong>
                                    </td>


                                    <td class='px-0'>
                                        <strong>{{ $book->name }}</strong>
                                    </td>
                                    
                                    {{-- <td class='px-0 text-center'>
                                        {{ $book->ISBN }}</td> --}}

                                    {{-- recent buyers --}}
                                    {{-- <td class='px-0 text-center'>
                                        @include('snit.auth.dashboard.books.partials.recentBuyers')
                                    </td> --}}

                                    {{-- pricing {hybrid} --}}
                                    <td class='px-0'>
                                        <span class="badge bg-label-primary me-1 text-xs">
                                            ₦{{ number_format($book->price, 2) }} | <span class="text-red-500"> ₦{{ number_format($book->eBookPrice(), 2) }}</span>
                                        </span>
                                    </td>
                                    {{-- profit section--}}

                                    {{-- hardcover profit --}}
                                    <td class='px-0'>
                                        @include('snit.auth.dashboard.books.partials.profitMarginHardCover')
                                    </td>

                                    {{-- ebook profit --}}
                                    <td class='px-0'>
                                        @include('snit.auth.dashboard.books.partials.profitMarginEbook')
                                    </td>

                                    {{-- total profit --}}

                                    <td class='px-0'>
                                        @include('snit.auth.dashboard.books.partials.profitMarginTotal')
                                    </td>
                                    {{-- edit book --}}
                                    <td class='px-2'>
                                        <a href="{{ route('books.edit', $book->id) }}">
                                            <i class="bx bx-edit-alt me-1"></i></a>
                                    </td>
                                    {{-- edit document file --}}
                                    <td class='px-2'>
                                        @if ($book->doc?->id)
                                            <a class="" href="{{ route('docs.edit', $book->doc->id) }}"><i
                                                    class="bx bx-edit me-1"></i></a>
                                        @else
                                            <a class='px-0 text-center'
                                                href="{{ route('docs.create', ['uploadfiles' => true, 'book' => $book]) }}"><i
                                                    class="bx bx-edit-alt me-1"></i>
                                                Add File </a>
                                        @endif
                                    </td>
                                    {{-- <td class='px-0 text-center'>
                                        <x-delete-refresh resourcename='book'
                                            :resource="$book"
                                            routename="books.destroy" />
                                    </td> --}}
                                </tr>
                            @endforeach
                        </tbody>
                    </table>
                </div>
            </div>
        </div>
    @endif
    <!--/ Bordered Table -->
@endsection
