@extends('snit.layouts.app')

@section('content')
    <!-- Bordered Table -->
    @if (isset($allRelatedClients) and count($allRelatedClients) >= 1)
        <div class="card mx-4 my-2">
            <h5 class="mx-4 mt-3 text-secondary"><small> Filter Results for
                "{{ $requestedDept->name }} {{ $requestedDept->dept_level }}"</small></h5>

            <div class="d-flex justify-content-between mt-0">
                <h5 class="mx-4">Showing My Clients for "{{ $requestedDept->name }} {{ $requestedDept->dept_level }}"
                </h5>
                <div class="d-block mx-5">
                    <div class="mt-0">
                        <p class="text-lg">
                            Total Purchases : {{ count($allRelatedClients) }}
                        </p>
                    </div>
                </div>
            </div>

            <div class="">
                @isset($searchfor)
                    {{-- {{ dd(request()->all()) }} --}}
                    <h5 class="mx-4 text-secondary">
                        <span> Search Results for "{{ $searchfor }}"</span>
                        <span>
                            <a href="{{ route('book-clients.show-all', $requestedDept->slug) }}">
                                <span class="text-gray-500 underline">Clear Search
                                    Results</span>
                            </a>
                        </span>
                    </h5>
                @endisset
            </div>

            <div class="mx-4 mb-3 text-sm">
                <a href="{{ route('dept-clients.index') }}" class="text-blue-500">
                    &lArr; back to departments

                </a>
            </div>
            <form class="">
                <div class="flex mx-4 col-10 col-sm-8 gap-x-3 justify-between">
                    <div class="book-title w-full">
                        <label for="" class="ms-1">Book Title</label>

                        <div class="input-group">
                            <select class="form-select" name="bookSlug" id="bookSlug">
                                @foreach ($booksSoldInDept as $book)
                                    <option @selected(request('bookSlug') == $book->slug) value="{{ $book->slug }}">{{ $book->name }}
                                    </option>
                                @endforeach
                            </select>
                            {{-- <button type="submit" class="btn btn-primary">Go <i class="bx bx-arrow-to-right"></i></button> --}}
                        </div>
                    </div>

                    <div class="w-full">
                        <label for="" class="ms-1">Purchase Type</label>

                        <div class="input-group">
                            <select class="form-select" name="purchaseType" id="purchaseType">
                                @foreach (['all', 'ebook', 'hardcover'] as $purchaseType)
                                    <option @selected(request('purchaseType') == $purchaseType)
                                        value="{{ $purchaseType == 'all' ? null : $purchaseType }}">
                                        {{ $purchaseType }}
                                    </option>
                                @endforeach
                            </select>
                            <button type="submit" class="btn btn-primary btn-sm">Go <i
                                    class="bx bx-arrow-to-right"></i></button>
                        </div>
                    </div>

                    <div class="w-full mt-4 mx-6 flex">
                        @if (request('purchaseType') or request('bookSlug'))
                            <a class="btn btn-sm btn-outline-primary"
                                href="{{ route('dept-clients.show-all', $requestedDept->slug) }} ">
                                View all
                            </a>
                        @endif
                    </div>

                </div>
            </form>

            <div class="card-body mt-10">
                <div class="table-responsive text-nowrap">
                    <table class="table  py-5">
                        <thead>
                            <tr class="bg-blue-800">
                                @foreach ($table_headers as $heading)
                                    @if (in_array($heading, $hideFromPrint))
                                        <th class="text-white m-0 p-0 print:hidden"><small>{!! $heading !!}</small>
                                        </th>
                                    @else
                                        <th class="text-white m-0 p-0"><small>{!! $heading !!}</small></th>
                                    @endif
                                @endforeach
                            </tr>
                        </thead>
                        <tbody>
                            @foreach ($allRelatedClients as $client)
                                <tr>
                                    <td class="m-0 px-0 py-2  print:hidden">
                                        <i class="fab fa-angular fa-lg text-danger mx-0"></i>
                                        <span class="capitalize"
                                            title="{{ $client->client_model->org->name }}">{{ $client->client_model->org->slug }}</span>
                                    </td>

                                    <td class="m-0 px-0 py-2">
                                        {{-- <i class="fab fa-angular fa-lg text-danger mx-0"></i> --}}
                                        <span class="capitalize">
                                            {{ $client->client_model->getFullName() }}
                                            <br>
                                            <span class="hidden print:block">{{ $client->client_model->mat_num }}</spanc>
                                            </span>
                                    </td>
                                    <td class="uppercase m-0 px-0 py-2 print:hidden">
                                        <span>{{ $client->client_model->mat_num }}</span>
                                    </td>

                                    {{-- phone --}}
                                    <td class=" my-2 px-0 py-2 print:hidden">
                                        <span>{{ $client->client_model->phone }}</span>
                                    </td>

                                    {{-- purchase date --}}
                                    <td class="my-2 px-0 py-2"><span class="mx-0">{{ $client->created_at_key }}</span>
                                    </td>

                                    {{-- book purchased --}}
                                    <td class="my-2 px-0 py-2 print:hidden">
                                        <span class="uppercase">{{ $client->itemPurchased->name }}</span>
                                    </td>

                                    {{-- purchase type --}}
                                    <td class="my-2 px-0 py-2 font-semibold">
                                        <span>{{ $client->itemPurchased->options->type }}</span>
                                    </td>

                                    {{-- amount --}}
                                    <td class=" my-2 px-0 py-2 print:hidden">
                                        <span>{{ $client->itemPurchased->options->original_price }}</span>
                                    </td>

                                    {{-- received status for student signature --}}
                                    <td class="m-0 p-1">
                                        <span class="font-semibold">
                                            {{ $client->itemPurchased->options->type == 'ebook' ? 'PDF ✅' : '' }}
                                        </span>
                                    </td>
                                </tr>
                            @endforeach
                        </tbody>
                    </table>
                </div>
            </div>
        </div>
    @elseif(isset($allRelatedClients) and count($allRelatedClients) < 1 && request('purchaseType'))
        <div class="my-5 py-5 text-center fw-light text-white text-sm">
            <p>You have no {{ request('purchaseType') }} Clients in
                <span class="uppercase block"> {{ $requestedDept->name }}</span>
                <span>that purchased</span>
                <span class="uppercase block"> "{{ getBookNameFromSlug(request('bookSlug')) }}"</span>
            </p>
            <p class="mt-4">
                <button class="btn btn-primary" onclick="history.back()">
                    Search again
                </button>
            </p>
        </div>
    @elseif(isset($allRelatedClients) and count($allRelatedClients) < 1 && !request('purchaseType'))
        <div class="my-5 text-center fw-light text-sm text-white">
            <span>You have no Clients in</span>
            <span class="uppercase block">{{ $requestedDept->name }}</span>
            <span>that purchased</span>
            <span class="uppercase"> "{{ getBookNameFromSlug(request('bookSlug')) }}"</span>


            <p class="mt-4">
                <button class="btn btn-primary" onclick="history.back()">
                    Search again
                </button>
            </p>

        </div>
    @endif
    <!--/ Bordered Table -->
@endsection
