@extends('snit.layouts.app')

@section('content')
    <!-- Bordered Table -->
    @if (count($allBooks) < 1)
        <h4 class="my-5 py-5 text-center fw-light"> You have no books <a class="underline" href="{{ route('books.create') }}">
                click here </a> to create a new book </h4>
    @else
        <div class="card m-4">
            <div class="d-flex justify-content-between">
                <h5 class="card-header">My Books</h5>
                <div class="d-block m-4">
                    <a href="{{ route('books.create') }}" class="menu-link">
                        <i class="menu-icon tf-icons bx bx-plus-circle"></i>
                        <div data-i18n="Toasts">Add New </div>
                    </a>
                    <div class="mt-3">
                        Total : {{ count($allBooks) }}
                    </div>
                </div>
            </div>

            <div class="card-body">
                <div class="table-responsive text-nowrap">
                    <table class="table table-bordered py-5">
                        <thead>
                            <tr>
                                @foreach ($table_headers as $heading)
                                    <th>{!! $heading !!}</th>
                                @endforeach
                            </tr>
                        </thead>
                        <tbody>
                            @foreach ($allBooks as $book)
                                <tr class="@empty($book->doc) bg-red-500  text-white @endempty">
                                    {{-- front cover image --}}
                                    <td>
                                        <img src="/storage/{{ $book->doc->front_cover ?? ' ' }}"
                                            alt="{{ $book->doc->front_cover ?? 'No Image' }}" class="src"
                                            width="100" />
                                    </td>

                                    {{-- back cover image --}}
                                    {{-- <td>
                                        <img src="/storage/{{ $book->doc->back_cover ?? ' ' }}"
                                            alt="{{ 'n/a' }}" class="src"
                                            width="100" />
                                    </td> --}}

                                    {{-- file --}}
                                    <td>
                                        {!! (Storage::disk('public')->exists($book->doc?->book_file) or  $book->doc?->book_file) ?
                                         ' <i class="bx bx-check-circle text-green-500"></i>' : '<i class="bx bx-x text-red-500"></i>' !!}
                                    </td>
                                    <td>
                                        <i class="fab fa-angular fa-lg text-danger me-3"></i>
                                        <strong>{{ $book->name }}</strong>
                                    </td>
                                    <td>{{ $book->ISBN }}</td>

                                    {{-- recent buyers --}}
                                    <td>
                                        <ul class="list-unstyled users-list m-0 avatar-group d-flex align-items-center mb-2">
                                            <li data-bs-toggle="tooltip" data-popup="tooltip-custom" data-bs-placement="top"
                                                class="avatar avatar-xs pull-up" title="Lilian Fuller">
                                                <img src="/static/assets/img/avatars/5.png" alt="Avatar"
                                                    class="rounded-circle" />
                                            </li>
                                            <li data-bs-toggle="tooltip" data-popup="tooltip-custom" data-bs-placement="top"
                                                class="avatar avatar-xs pull-up" title="Sophia Wilkerson">
                                                <img src="/static/assets/img/avatars/6.png" alt="Avatar"
                                                    class="rounded-circle" />
                                            </li>
                                            <li data-bs-toggle="tooltip" data-popup="tooltip-custom" data-bs-placement="top"
                                                class="avatar avatar-xs pull-up" title="Christina Parker">
                                                <img src="/static/assets/img/avatars/7.png" alt="Avatar"
                                                    class="rounded-circle" />
                                            </li>
                                        </ul>

                                        {{-- <a href="{{ route('clients.show', ['book' => $book->id]) }}">view all</a> --}}
                                    </td>
                                    <td><span class="badge bg-label-primary me-1">{{ number_format($book->price, 2) }}</span></td>

                                    {{-- edit book --}}
                                    <td>
                                        <div class="d-flex">
                                            <a class="dropdown-item" href="{{ route('books.edit', $book->id) }}"><i
                                                    class="bx bx-edit-alt me-1"></i></a>
                                            {{-- <a class="dropdown-item" href="{{ route('books.edit', $book->id) }}"><i
                                          class="bx bx-edit-alt me-1"></i></a> --}}
                                        </div>
                                    </td>
                                    {{-- edit doc file --}}
                                    <td>
                                        @if ($book->doc?->id)
                                            <a class="dropdown-item" href="{{ route('docs.edit', $book->doc->id) }}"><i
                                                    class="bx bx-edit me-1"></i></a>
                                        @else
                                            <a class="dropdown-item"
                                                href="{{ route('docs.create', ['uploadfiles' => true, 'book' => $book]) }}"><i
                                                    class="bx bx-edit-alt me-1"></i> Add File </a>
                                        @endif
                                    </td>
                                    <td>
                                        <x-delete-refresh resourcename='book' :resource="$book" routename="books.destroy" />
                                    </td>
                                </tr>
                            @endforeach
                        </tbody>
                    </table>
                </div>
            </div>
        </div>
    @endif
    <!--/ Bordered Table -->
@endsection
