@extends('snit.layouts.app')

@section('content')
    <!-- Bordered Table -->
    @if (count($allBooks) < 1)
        <p class="my-5 py-5 text-sm text-center text-white fw-light">
            Oops! You have no books <br><br>
            <a class="underline text-sm text-blue-400" href="{{ route('books.create') }}">
                &rArr; Add Your First Book
            </a>
        </p>
    @else
        <div class="card m-4">
            <div class="">
                <div class="card-header text-center">
                    <h1 class="h6 my-0 uppercase">
                        Income (Profit) Report</h1>
                </div>
                <div class="mx-4 text-right my-0 text-xs">
                    <p class="text-black my-0">Total Books Published: {{ count($allBooks) }}</p>
                    <p class="text-primary my-0">
                        Total Sales Generated: ₦ {{ money(author()->totalSalesFromBooks()) }}
                    </p>

                    <p class="text-success">
                        Total Profit Generated: ₦ {{ money(author()->presentTotalProfit()) }}
                    </p>
                </div>
            </div>

            <div class="card-body">
                <div class="table-responsive text-nowrap">
                    <table class="table py-5">
                        {{-- profit margins --}}
                        <thead>
                            <tr class="bg-primary">
                                @foreach ($table_headers as $heading)
                                    <th class='max-w-fit text-xs ps-1 '>
                                        <small class="text-white">{!! $heading !!}</small>
                                    </th>
                                @endforeach
                            </tr>
                        </thead>
                        <tbody>
                            @foreach ($allBooks as $book)
                                <tr
                                    class="@empty($book->doc) bg-red-500  text-white @else
                                        hover:bg-blue-50 hover:text-black hover:transition-all cursor-pointer
                                     @endempty
                                    ">

                                    {{-- front cover image --}}
                                    <td class='px-0 '>
                                        <img src="/storage/{{ $book->doc->front_cover ?? ' ' }}" width=40
                                            alt="{{ $book->doc->front_cover ?? 'No Image' }}" class="src" />
                                    </td>


                                    {{-- book name --}}
                                    <td class='px-0'>
                                        <span class="text-xs">{{ $book->name }}</span>
                                    </td>

                                    {{-- pricing --}}
                                    <td class='px-0'>
                                        <span class="badge bg-label-primary me-1 text-xs">
                                            ₦{{ number_format($book->price, 2) }} | <span class="text-red-500">
                                                ₦{{ number_format($book->eBookPrice(), 2) }}</span>
                                        </span>
                                    </td>

                                    {{-- hardcover profit --}}
                                    <td class='px-0'>
                                        @include('snit.auth.dashboard.books.partials.profitMarginHardCover')
                                    </td>

                                    {{-- ebook profit --}}
                                    <td class='px-0'>
                                        @include('snit.auth.dashboard.books.partials.profitMarginEbook')
                                    </td>

                                    {{-- total profit --}}

                                    <td class='px-0'>
                                        @include('snit.auth.dashboard.books.partials.profitMarginTotal')
                                    </td>
                                </tr>
                            @endforeach
                        </tbody>
                    </table>
                </div>
            </div>
        </div>
    @endif
    <!--/ Bordered Table -->
@endsection
