@extends('snit.layouts.app')

@section('content')
    <!-- Bordered Table -->
    @if (count($allBooks) < 1)
        <h4 class="my-5 py-5 text-center fw-light"> You have no books <a class="underline" href="{{ route('books.create') }}">
                click here </a> to create a new book </h4>
    @else
        <div class="card m-4">
            <div class="">
                <div class="card-header text-center">
                    <h1 class="h6 my-0 uppercase">
                        Revenue (Sales) Report</h1>
                </div>
                <div class="mx-4 text-right my-0 text-xs">
                    <p class="text-black my-0">Total Books Published: {{ count($allBooks) }}</p>
                    <p class="text-primary my-0">
                        Total Sales Generated: ₦ {{ number_format(author()->totalSalesFromBooks()) }}
                    </p>

                    <p class="text-success">
                        Total Profit Generated: ₦ {{ number_format(author()->presentTotalProfit()) }}
                    </p>

                </div>
            </div>

            <div class="card-body">
                <div class="table-responsive text-nowrap">
                    <table class="table">
                        <thead>
                            <tr class="bg-primary">
                                @foreach ($table_headers as $heading)
                                    <th><small><small class="text-white">{!! $heading !!}</small></small></th>
                                @endforeach
                            </tr>
                        </thead>
                        <tbody>
                            @foreach ($allBooks as $book)
                                <a href="#" style='pointer:cursor;'>
                                    <tr class="@empty($book->doc) bg-red-500  text-white @endempty">

                                        {{-- front cover image --}}
                                        <td>
                                            <img src="/storage/{{ $book->doc->front_cover ?? ' ' }}"
                                                alt="{{ $book->doc->front_cover ?? 'No Image' }}" class="src"
                                                width="50" />
                                        </td>


                                        <td>
                                            <i class="fab fa-angular fa-lg text-danger me-3"></i>
                                            <strong>{{ $book->name }}</strong>
                                        </td>

                                        {{-- general pricing  --}}
                                        <td>
                                            <span class="badge text-black me-1 text-xs">
                                                ₦{{ money($book->price) }} | <span class="text-violet-600">
                                                    ₦{{ money($book->eBookPrice()) }}</span>
                                            </span>
                                        </td>

                                        {{-- total hardcover books sold --}}

                                        <td>
                                            <span
                                                class="badge text-black text-xs fw-light me-1">{{ money($book->service->totalHardcoverOrders()->count()) }}</span>
                                        </td>

                                        {{-- total ebooks sold --}}
                                        <td><span
                                                class="badge text-black text-xs fw-light me-1">{{ money($book->service->totalEbookOrders()->count()) }}</span>
                                        </td>

                                        {{-- total books sold --}}
                                        <td><span
                                                class="badge text-black text-xs fw-light me-1">{{ money($book->service->getTotalNumberSold()) }}</span>
                                        </td>
                                        {{-- total income generated --}}
                                        <td><span class="text-sm me-1">
                                                {{ '₦' . money($book->presentTotalSalesGenerated()) }}</span>
                                        </td>
                                    </tr>
                                </a>
                            @endforeach
                        </tbody>
                    </table>
                </div>
            </div>
        </div>
    @endif
    <!--/ Bordered Table -->
@endsection
