<?php use App\Enums\UserRoleEnum; ?>
@extends('snit.layouts.auth')

@section('content')
    <h4 class="mb-2">Adventure starts here 🚀</h4>
    <p class="mb-4 text-xs">Make your app management easy and fun!</p>

    <form id="formAuthentication" class="w-full" action="{{ route('register') }}" method="POST">
        @csrf
        <div class="flex gap-1">

            <div class="mb-3 w-full">
                <label for="name" class="my-0">Username <i class="text-red-500">*</i></label>
                <input required type="text" value='{{ old('name') }}' class="form-control" id="name"
                    name="name" placeholder="Choose a username" autofocus />
                <x-input-single-error name="name" />
            </div>

            <div class="mb-3 w-full">
                <label for="email" class="my-0">Email <i class="text-red-500">*</i></label>
                <input required type="email" value='{{ old('email') }}' class="form-control" id="email"
                    name="email" placeholder="Enter your best email" />
                <x-input-single-error name="email" />
            </div>
        </div>

        {{-- organisation box --}}
        <div class="mb-3 ">
            <label for="organization" class="my-0">
                Organization <i class="text-red-500">*</i></label>
            <br>
            <small class="text-info text-xs">
                <small class="block leading-5"> If your institution is not
                    available, kindly select the last option</small>
            </small>

            <select id="organisation" name="organisation" class="select2 form-select">
                <option value="">Select Organisation / Institution</option>
                @foreach ($orgs as $org)
                    <option value="{{ $org->id }}">{{ $org->name }}</option>
                @endforeach
                {{-- <option value="new" class='text-danger font-bold'>I can't find my
                    Organisation / Institution
                </option> --}}
            </select>

            <input type="text" class="form-control d-none" id="organisation_name" name="organisation_name"
                value="{{ old('organisation_name') }}" placeholder="Enter  Your Organisation's full name" />

            <x-input-single-error name="organisation" />
        </div>
        {{-- eol organisation box --}}

        {{-- role box --}}
        <div class="mb-3 ">
            <label for="role" class="my-0">Role <i class="text-red-500">*</i></label>
            <br>
            <select required id="role" name="role" class="select2 form-select">
                <option value="" class='uppercase'>What best describes you</option>
                @foreach (['instructor' => 'Instructor (I\'m here to sell)', 'org_admin' => 'Admin (I\'m here for my institution )'] as $key => $org)
                    <option class='' value="{{ $key }}">
                        {{ strtoupper($org) }}</option>
                @endforeach
            </select>
            <x-input-single-error name="role" />
        </div>
        {{-- eol role box --}}



        {{-- password box --}}
        <div class="flex gap-1">
            <div class="mb-3 form-password-toggle">
                <label class="my-0" for="password">Password <i class="text-red-500">*</i></label>
                <div class="input-group input-group-merge">
                    <input required type="password" value='{{ old('password') }}' id="password" class="form-control"
                        name="password"
                        placeholder="&#xb7;&#xb7;&#xb7;&#xb7;&#xb7;&#xb7;&#xb7;&#xb7;&#xb7;&#xb7;&#xb7;&#xb7;"
                        aria-describedby="password" />
                    <span class="input-group-text cursor-pointer"><i class="bx bx-hide"></i></span>
                </div>
                <x-input-single-error name="password" />
            </div>

            {{-- password confirmation field --}}
            <div class="mb-3 form-password-toggle">
                <label required class="my-0" for="password_confirmation">
                    Confirm Password <i class="text-red-500">*</i>
                </label>
                <div class="input-group input-group-merge">
                    <input type="password" value='{{ old('password_confirmation') }}' id="password_confirmation"
                        class="form-control" name="password_confirmation"
                        placeholder="Confirm password &#xb7;&#xb7;&#xb7;&#xb7;&#xb7;&#xb7;&#xb7;&#xb7;&#xb7;&#xb7;&#xb7;&#xb7;"
                        aria-describedby="password_confirmation" />
                    <span class="input-group-text cursor-pointer"><i class="bx bx-hide"></i></span>
                </div>
                <x-input-single-error name="password_confirmation" />
            </div>
            {{-- end of pwd --}}
        </div>

        <div class="mb-3">
            <div class="form-check">
                <div class="">
                    {{-- toc --}}

                    <label class="form-check-label cursor-pointer" for="terms-conditions">
                        <input class="form-check-input" type="checkbox" id="terms-conditions" name="terms" required />
                        I agree to
                        <a href="javascript:(0)">privacy policy & terms </a>
                    </label>
                </div>


                {{-- is school admin --}}
                <div class="my-3 d-none" id="{{ UserRoleEnum::ORG }}_box">
                    <label class="form-check-label cursor-pointer" for="{{ UserRoleEnum::ORG }}">
                        <input class="form-check-input" type="checkbox" id="{{ UserRoleEnum::ORG }}"
                            name="{{ UserRoleEnum::ORG }}" />
                        <small class="text-gray-400">
                            Admin (For Institution Admins Only) <i class="text-red-500">*</i></small>
                    </label>
                    <x-input-single-error name="terms" />
                </div>


                {{-- ims code input field --}}
                <div class="mb-3 sch_ims_code_box d-none">
                    <label for="name" class="my-0">School IMS
                        Code</label>
                    <input type="text" value='{{ old('sch_ims_code') }}' class="form-control" id="sch_ims_code"
                        name="sch_ims_code" placeholder="Enter your institution's Unique IMS Code" autofocus />
                    <x-input-single-error name="sch_ims_code" />
                </div>

            </div>
        </div>
        <button class="btn btn-primary d-grid w-100">Sign up</button>
    </form>

    <p class="text-center mt-4">
        <span>Already have an account?</span>
        <a href="{{ route('login') }}">
            <span>Sign in instead</span>
        </a>
    </p>
@endsection

@section('page_css')
    <style>
        .schCodeBox label,
        .schCodeBox input#sch_ims_code,
        .schCodeBox:active,
        .org_admin:checked {
            transition: all 3s;
        }
    </style>
@endsection
@push('page_scripts')
    <script src="/page_scripts/registerPage.js"></script>
@endpush
