@php
    use Gloudemans\Shoppingcart\Facades\Cart;
@endphp
@extends('snit.layouts.app')

@section('content')
    <!-- Bordered Table -->
    <div class="card m-4">
        <div class="d-flex justify-content-between">
            <p class="card-header">My Cart</p>
            <div class="d-block m-4">
                <div class="mt-3">
                    Total : {{ Cart::content()->count() }}
                </div>

                <form class="mt-3" action="{{ route('guest.cart-destroy') }}" method="post">@method('delete') @csrf
                    <button type="submit" class="btn-danger">Empty cart</button>
                </form>
            </div>
        </div>

        <div class="card-body">
            <div class="table-responsive text-nowrap">
                @if (Cart::content()->count() > 0)
                    <table class="table py-5">
                        <thead>
                            <tr class="">
                                @foreach ($table_headers as $heading)
                                    <th class="mx-0 px-0 text-left"> <small>{!! $heading !!}</small></th>
                                @endforeach
                            </tr>
                        </thead>
                        <tbody>
                            @foreach (Cart::content() as $cartItem)
                                <tr class="@empty($cartItem->name) bg-red-500  text-white @endempty">
                                    {{-- front cover image --}}
                                    <td class="px-0 mx-0 text-left">
                                        <img src="/storage/{{ $cartItem->model->doc->front_cover ?? ' ' }}"
                                            alt="{{ $cartItem->model->doc->front_cover ?? 'No Image' }}" class="src"
                                            width="50" />
                                    </td>

                                    {{-- book title or name --}}
                                    <td class="mx-0 px-0 text-left w-min">
                                        <i class="fab fa-angular fa-lg text-danger"></i>
                                        <strong class="text-xs">{{ $cartItem->name }}</strong>
                                    </td>

                                    {{-- book author --}}
                                    <td class="mx-0 px-0 text-left w-min">
                                        <i class="fab fa-angular fa-lg text-danger"></i>
                                        <strong
                                            class="text-xs">{{ $cartItem->options->book_author->presentFullName() }}</strong>
                                    </td>

                                    <td class="mx-0 px-0 text-left w-min">
                                        <i class="fab fa-angular fa-lg text-danger"></i>
                                        <strong
                                            class="text-xs">{{ $cartItem->options->book_author->getOrg('slug') }}</strong>
                                    </td>


                                    {{-- book qty --}}
                                    <td class="mx-0 px-0 text-left w-min"><span class="text-xs">{{ $cartItem->qty }}</span>
                                    </td>

                                    {{-- type --}}
                                    <td class="mx-0 px-0 text-left w-min"><span
                                            class="badge text-xs bg-label-primary">{{ $cartItem->options->type }}</span>
                                    </td>


                                    {{-- book price --}}
                                    <td class="mx-0 px-0 text-left ">
                                        {{-- for computation (qty update) --}}
                                        <span class="d-none">{{ $cartItem->price }}</span>
                                        {{-- for presentation --}}
                                        <span class="me-1 text-xs">{{ money($cartItem->price) }}</span>

                                    </td>

                                    {{-- remove item from cart  --}}
                                    <td>
                                        <form id="remove_from_cart_form_{{ $cartItem->id }}"
                                            action="{{ route('guest.cart-destroy', ['item_id' => $cartItem->id]) }}"
                                            method="post">@csrf @method('delete')
                                            <button type="submit"
                                                class='bx bx-trash text-danger border-none bg-transparent'>

                                            </button>
                                        </form>
                                    </td>
                                </tr>
                            @endforeach
                        </tbody>
                    </table>

                    <div class=" bg-blue-200 w-min p-3 mt-2 rounded">
                        <div class="flex sub-total-col gap-3 mt-1 ml-1 p-2  w-min border-bottom border-2">
                            <div class="key">
                                SubTotal
                            </div>
                            <div class="val">
                                ₦ {{ Cart::subTotal() }}
                            </div>

                        </div>

                        <div class="flex sub-total-col gap-3 ml-1 p-2  w-min  border-2">
                            <div class="key">
                                Tax
                            </div>
                            <div class="val">
                                ₦ {{ Cart::tax() }}
                            </div>
                        </div>

                        <div class="flex sub-total-col gap-3  ml-1 p-2 border-top border-2 w-min ">
                            <div class="key">
                                Total
                            </div>
                            <div class="val">
                                ₦ {{ Cart::total() }}
                            </div>
                        </div>



                        <div class="flex sub-total-col gap-3 mt-1">
                            <a class="inline d-inline btn btn-outline-primary text-xs" title="Proceed to Checkout"
                                sr-only="Proceed to Checkout" href="{{ route('guest.choose-client') }}">
                                Proceed to Checkout
                            </a>
                        </div>
                    </div>
                @else
                    <p class="text-center text-lg fw-light">Cart is empty ...
                        <a href="{{ route('guest.bookstore') }}"> Go Shopping</a>
                    </p>
                @endif
            </div>
        </div>
    </div>
    <!--/ Bordered Table -->
@endsection
