<?php

use App\Services\RegistrationTestService;
use App\Providers\RouteServiceProvider;
use Illuminate\Foundation\Testing\RefreshDatabase;

uses(RefreshDatabase::class);

// for debugging
// $this->withoutExceptionHandling();
// dump($response->getStatusCode());
// dump($response->getContent());

// global variables
$regTestService = new RegistrationTestService;

test('registration screen can be rendered', function () {

    $response = $this->get('/register');

    $response->assertStatus(200);
});

/*
|--------------------------------------------------------------------------
| POSITIVES.
|--------------------------------------------------------------------------
*/

// Registration with Existing Organisations
test(
    'new users can register as instructors for existing organisation',
    function () use ($regTestService) {
        $formData = $regTestService->createInstructorForExistingOrgs();

        $response = $this->post('/register', $formData);

        $this->assertAuthenticated();
        $response->assertRedirect(RouteServiceProvider::HOME);
    }
);

// Registration with New Organisations
test(
    'new users can register as instructors for new organisations',
    function () use ($regTestService) {
        $formData = $regTestService->createInstructorForNewOrgs();
        $response = $this->post('/register', $formData);

        $this->assertAuthenticated();
        $response->assertRedirect(RouteServiceProvider::HOME);
    }
);

// OrgAdmin Registration with Existing Organisations

test(
    'new users can register as org_admins with existing organisations',
    function () use ($regTestService) {

        $formData = $regTestService->createOrgAdminForExistingOrgs();

        $response = $this->post('/register', $formData);
        // dd($formData);
    
        $this->assertAuthenticated();
        $response->assertRedirect(RouteServiceProvider::SCH_HOME);
    }
);


test(
    'new users can register as org_admins with new organisations',
    function () use ($regTestService) {

        $formData = $regTestService->createOrgAdminForNewOrgs();

        $response = $this->post('/register', $formData);
    
        $this->assertAuthenticated();
        $response->assertRedirect(RouteServiceProvider::SCH_HOME);
    }
);


/*
|--------------------------------------------------------------------------
| NEGATIVES.
|--------------------------------------------------------------------------
*/

test(
    "new users can't register as admins from the registration form",
    function () use ($regTestService) {

        $formData = $regTestService->createAppAdmin(); //Arrange
    
        $response = $this->post(route('register.store'), $formData); //Action
    
        // Assert
        $this->assertFalse(auth()->check());
        $this->assertGuest();
        $response->assertStatus(302);
        $response->assertRedirect(route('register'));

        // $response->assertRedirect(RouteServiceProvider::HOME);
    }
);


